@@ -1,4 +1,4 @@
-// $Id: Cxx.cc,v 1.76 2002/02/25 14:58:11 christof Exp $
+// $Id: Cxx.cc,v 1.102 2003/05/17 18:22:09 christof Exp $
 /*  glade--: C++ frontend for glade (Gtk+ User Interface Builder)
  *  Copyright (C) 1998  Christof Petig
  *  Copyright (C) 1999-2000 Adolf Petig GmbH & Co. KG, written by Christof Petig
@@ -26,8 +26,10 @@
 
 // for some accesses
 #include <time.h>
+#ifndef __MINGW32__
 #include <pwd.h>
 #include <sys/utsname.h>
+#endif
 #include <unistd.h>
 #include <sys/stat.h>
 
@@ -52,7 +54,7 @@ void Cxx::CreateSupportFiles()
       support_c.Include("glademm_support"+Configuration.header_suffix,true);
       
    if (Configuration.lookup_table)
-   {  support_h.Include("gtk--/widget.h");
+   {  support_h.Include(Configuration.GtkmmIncludePath()+"widget.h");
       
       support_h.Declaration().Funct_ReturnType(WriterBase::GtkPrefix()+"Widget *")
       		.FunctionName("glademm_get_Widget")
@@ -113,29 +115,23 @@ void Cxx::WriteTags()
    if (!Configuration.bare_bones)
    {  main.open(FileNameByType(File_MAIN_CC)); WriteHeader(main,File_MAIN_CC);
       std::string fname(FileNameByType(File_MAKEFILE));
-      if (Configuration.no_autoconf || access(fname.c_str(),F_OK))
+      if (Configuration.no_autoconf)
       {  makefile.open(fname); WriteHeader(makefile,File_MAKEFILE);
       }
       if (!Configuration.no_autoconf)
       {  autogen_name=FileNameByType(File_AUTOGEN_SH);
-         if (access(autogen_name.c_str(),F_OK))
-         {  autogen.open(autogen_name,std::ios::out); 
-      	    WriteHeader(autogen,File_AUTOGEN_SH);
-      	 }
+         autogen.open(autogen_name,std::ios::out); 
+	 WriteHeader(autogen,File_AUTOGEN_SH);
       	 
          makefile_am.open(FileNameByType(File_MAKEFILE_AM)); 
          WriteHeader(makefile_am,File_MAKEFILE_AM);
          
-         fname=FileNameByType(File_CONFIGURE_IN);
-         if (access(fname.c_str(),F_OK))
-         {  configure_in.open(fname);
-            WriteHeader(configure_in,File_CONFIGURE_IN);
-         }
-         fname=FileNameByType(File_toplevel_MAKEFILE_AM);
-         if (access(fname.c_str(),F_OK))
-         {  toplev_Make_am.open(fname);
-            WriteHeader(toplev_Make_am,File_toplevel_MAKEFILE_AM);
-         }
+         configure_in.open(FileNameByType(File_CONFIGURE_IN));
+	 WriteHeader(configure_in,File_CONFIGURE_IN);
+
+         toplev_Make_am.open(FileNameByType(File_toplevel_MAKEFILE_AM));
+	 WriteHeader(toplev_Make_am,File_toplevel_MAKEFILE_AM);
+
          fname=FileNameByType(File_README);
          if (access(fname.c_str(),F_OK))
          {  readme.open(fname);
@@ -161,11 +157,8 @@ void Cxx::WriteTags()
             WriteHeader(todo,File_TODO);
          }
          
-         fname=FileNameByType(File_ACCONFIG_H);
-         if (access(fname.c_str(),F_OK))
-         {  acconfig.open(fname);
-            WriteHeader(acconfig,File_ACCONFIG_H);
-         }
+         acconfig.open(FileNameByType(File_ACCONFIG_H));
+	 WriteHeader(acconfig,File_ACCONFIG_H);
          
          if (Configuration.gettext_support)
          {  po_cl.open(FileNameByType(File_po_ChangeLog,0),std::ios::app);
@@ -208,7 +201,7 @@ void Cxx::WriteTags()
 void Cxx::WriteTags(Tag &t)
 {  // 2do: make names unique?
    if (t.Type()!="widget")
-   {  for(Tag::iterator i=t.begin();i!=t.end();++i) 
+   {  FOR_EACH_TAG(i,t)
          WriteTags(*i);
    }
    else
@@ -225,22 +218,24 @@ void Cxx::ApplyPreferences(Tag &t,int level)
 
    WidgetMap[w.Name()]=&t;
    if (!level)
-   {  bool seperate_class=t.getBool(CXX_SEPERATE_CLASS),
-   	   seperate_file=t.getBool(CXX_SEPERATE_FILE);
+   {  bool seperate_class=w.getBoolProperty(CXX_SEPERATE_CLASS),
+   	   seperate_file=w.getBoolProperty(CXX_SEPERATE_FILE);
       if (!seperate_class || !seperate_file)
       {  // std::cerr << w.Name() << ": toplevel widgets have their own class & file, marked\n";
          if (!seperate_class)
-         {  w.mark(CXX_SEPERATE_CLASS);
+         {  w.setProperty(CXX_SEPERATE_CLASS);
             seperate_class=true;
          }
          if (!seperate_file)
-         {  w.mark(CXX_SEPERATE_FILE);
+         {  w.setProperty(CXX_SEPERATE_FILE);
             seperate_file=true;
          }
       }
    }
-   LookupWriter(w).ApplyPreferences(t);
-   if (level && t.getBool(CXX_SEPERATE_FILE) && !t.getBool(CXX_SEPERATE_CLASS))
+   // special exception:
+   // I do not want the LibGlademm Object, so I do not use LookupWriter(w)
+   LookupWriter(w.Class(),false).ApplyPreferences(t);
+   if (level && w.getBoolProperty(CXX_SEPERATE_FILE) && !w.getBoolProperty(CXX_SEPERATE_CLASS))
    {  std::cerr << w.Name() << "Code error (" << w.Class() << ")\n";
    }
   
@@ -275,11 +270,21 @@ void Cxx::WriteHeader(CxxFile &f,File_type tp)
    	 f << Configuration.FileName("",tp,File_GLADE|File_NODIR) << "\n"
    	 	"\n"
 		"// This file is for your program, I won't touch it again!\n\n";
+	 f.Include("config.h"); // this is always a good idea
 	 if (!Configuration.gnome_support)
-	    f.Include("gtk--/main.h");
+	    f.Include(Configuration.GtkmmIncludePath()+"main.h");
 	 else 
-	 {  f.Include("config.h");
-	    f.Include("gnome--/main.h");
+	 {  // We could be lazy and include libgnomeuimm.h...
+	    f.Include(Configuration.GnomemmIncludePath()+"main.h");
+	    // For module_info_get()
+	    if (GNOME2) {
+	       f.Include(Configuration.GnomeUImmIncludePath()+"init.h");
+	       // FIXME: This is a kludge. Until the day libgnomeuimm
+	       // properly initializes bonobouimm (there's a FIXME in
+	       // the source there), we need to call wrap_init()
+	       // ourself.
+	       f.Include("bonobomm/widgets/wrap_init.h");
+	    }
 	    if (Configuration.gettext_support)
 	       f.Include("libgnome/gnome-i18n.h");
 	 }
@@ -296,6 +301,9 @@ void Cxx::WriteHeader(CxxFile &f,File_type tp)
 		" * These entries are sometimes used by macros\n"
 		" * which glade-- uses.\n"
 		" */\n";
+	 f <<   "#undef PACKAGE\n"
+	 	"#undef VERSION\n"; // can't hurt for older versions
+	 	// needed for automake1.5 ?
 	 if (Configuration.gettext_support)
       	    f <<"#undef ENABLE_NLS\n"
 		"#undef HAVE_CATGETS\n"
@@ -303,7 +311,8 @@ void Cxx::WriteHeader(CxxFile &f,File_type tp)
 		"#undef HAVE_LC_MESSAGES\n"
 		"#undef HAVE_STPCPY\n"
 		"#undef HAVE_LIBSM\n"
-		"#undef PACKAGE_LOCALE_DIR\n";
+		"#undef PACKAGE_LOCALE_DIR\n"
+		"#undef GETTEXT_PACKAGE\n";
 	 // CHECKME: is this correct?
 	 if (Configuration.gnome_support)
 	    f <<"#undef PACKAGE_DATA_DIR\n"
@@ -317,58 +326,19 @@ void Cxx::WriteHeader(SystemFile &f,File_type tp)
 {  if (!f.good()) return;
    switch(tp)
    {  
-      case File_MAKEFILE:
+      case File_MAKEFILE:  
          WriteCreation(f,tp);
    	 f << "#\n# newer (non customized) versions of this file go to ";
    	 f << Configuration.FileName("",tp,File_GLADE|File_NODIR) << '\n';
    	 if (Configuration.no_autoconf)
-   	 {  std::string ENABLE_NLS;
-   	    if (Configuration.gettext_support) ENABLE_NLS="-DENABLE_NLS ";
-   	    if (Configuration.use_libglade)
-   	    {  std::string gnome(Configuration.gnome_support?"gnome":"");
-   	       std::string lgnomemm(Configuration.gnome_support?"-lgnomemm":"");
-   	       f << "\n"
-   	         "CXXFLAGS=$(CFLAGS) "<< ENABLE_NLS <<"$(shell gtkmm-config --cflags) $(shell libglade-config --cflags "<<gnome<<")\n"
-                 "LIBS=$(shell echo $$LIBS) $(shell gtkmm-config --libs) "<<lgnomemm<<" $(shell libtool-config --libs "<<gnome<<')';
-            }
-   	    else if (Configuration.gnome_support)
-   	       f << "\n"
-                 "CXXFLAGS=$(CFLAGS) "<< ENABLE_NLS <<"$(shell gtkmm-config --cflags) $(shell gnome-config --cflags gnomemm) $(shell gnome-config --cflags gnomeui)\n"
-                 "LIBS=$(shell echo $$LIBS) $(shell gtkmm-config --libs) -lgnomemm $(shell gnome-config --libs gnomemm)  $(shell gnome-config --libs gnomeui)";
-   	    else
-   	       f << "\n"
-                 "CXXFLAGS=$(CFLAGS) $(shell gtkmm-config --cflags)\n"
-                 "LIBS=$(shell echo $$LIBS) $(shell gtkmm-config --libs)";
-            f << "\n"
-                 "# calculate dependancies\n"
-                 "\n"
-                 "all: dep_all\n"
-                 "\n"
-                 "ifeq (.depend,$(wildcard .depend))\n"
-                 "include .depend\n"
-                 "dep_all: " << Configuration.main_filename << "\n"
-                 "else\n"
-                 "dep_all: .depend\n"
-                 "\tmake all\n"
-                 "endif\n"
-                 "\n"
-                 ".depend depend dep:\n"
-                 "\tfor i in *.cc ; do $(CXX) $(CXXFLAGS) -M $$i; done > .depend\n"
-                 "\n" << Configuration.main_filename << ": " 
-                 << FileName("",File_MAIN_CC,File_NODIR|File_OBJECT) 
-                 << ' ';
-   	 }
-   	 else
-   	 {  f << "# this file will overwrite itself !\n"
+   	    std::cerr << "non automake option has been removed!\n";
+   	 f << "# this file will overwrite itself !\n"
    	 	 "\n"
    	 	 "all:\n"
    	 	 "\t./autogen.sh\n";
-   	 }
          break;
       case File_MAKEFILE_AM: // in the src directory
          WriteCreation(f,tp);
-         if (Configuration.use_libglade)
-         {  std::cerr << "glade-- can't output a Makefile.am for libglade, yet\n"; }
    	 f << "#\n# newer (non customized) versions of this file go to ";
    	 f << FileName("",tp,File_GLADE|File_NODIR) << "\n"
    	      "bin_PROGRAMS = "<< Configuration.main_filename << "\n\n";
@@ -390,11 +360,12 @@ void Cxx::WriteHeader(SystemFile &f,File_type tp)
 		"AM_INIT_AUTOMAKE("<< Configuration.main_filename <<", 0.0)\n"
 	        "AM_CONFIG_HEADER(config.h)\n"
 	        "\n";
-	if (Configuration.gnome_support)
-	   f << "dnl Pick up the Gnome macros.\n"
-	   	"AM_ACLOCAL_INCLUDE(macros)\n"
-	   	"AM_MAINTAINER_MODE\n" // don't ask me why it needs this
-	   	"\n";
+	if (Configuration.gnome_support && GNOME1) 
+	{    f << "dnl Pick up the Gnome macros.\n"
+		 "AM_ACLOCAL_INCLUDE(macros)\n"
+		 "AM_MAINTAINER_MODE\n" // needed!
+		 "\n";
+	}
 	f <<    "AC_ISC_POSIX\n"
 		"AC_PROG_CC\n"
 		"AM_PROG_CC_STDC\n"
@@ -412,38 +383,70 @@ void Cxx::WriteHeader(SystemFile &f,File_type tp)
 	   	"GNOME_COMMON_INIT\n"
 	   	"GNOME_COMPILE_WARNINGS\n";
 	if (Configuration.gettext_support)
+	{  f << "dnl *************************************************\n"
+		"dnl gettext support\n"
+		"dnl *************************************************\n\n";
+	   // as seen in glade-2
+	   f << "GETTEXT_PACKAGE=" << Configuration.main_filename << '\n';
+	   f << "AC_SUBST(GETTEXT_PACKAGE)\n"
+	   	"AC_DEFINE_UNQUOTED(GETTEXT_PACKAGE,\"$GETTEXT_PACKAGE\")\n\n";
 	   f << "dnl Add the languages which your application supports here.\n"
-	   	"ALL_LINGUAS=\"\"\n"
-	   	"AM_"<<(!Configuration.gnome_support?"GNU":"GNOME")<<"_GETTEXT\n"
+	   	"ALL_LINGUAS=\"\"\n";
+	   f << "AM_";
+	   if (Configuration.gtk_version<Pkg_Version(1,3,0))
+	      f << (!Configuration.gnome_support?"GNU":"GNOME");
+	   else // glib 2.0
+	      f << /*"GLIB_"*/ "GNU";
+	   f <<"_GETTEXT\n"
 	   	"\n";
-        if (Configuration.gettext_support) // && !Configuration.gnome_support)
-		// is this correct?	
-             f << "dnl Set PACKAGE_LOCALE_DIR in config.h.\n"
+		// is this correct?
+		// only needed for gnome 1.x?
+           f << "dnl Set PACKAGE_LOCALE_DIR in config.h.\n"
    		"if test \"x${prefix}\" = \"xNONE\"; then\n"
    		"  AC_DEFINE_UNQUOTED(PACKAGE_LOCALE_DIR, \"${ac_default_prefix}/${DATADIRNAME}/locale\")\n"
    		"else\n"
    		"  AC_DEFINE_UNQUOTED(PACKAGE_LOCALE_DIR, \"${prefix}/${DATADIRNAME}/locale\")\n"
    		"fi\n"
 		"\n";
+        }
 	   f << "AC_LANG_CPLUSPLUS\n"
 		"\n";
 	if (!Configuration.gnome_support) // we trust ...
-	   f << "AM_PATH_GTKMM("<<Configuration.gtkmm_version.mymajor <<'.'
-	  << Configuration.gtkmm_version.myminor<<'.'
-	  << Configuration.gtkmm_version.mymicro<<",,\n"
-	     "        AC_MSG_ERROR(Cannot find GTK--: Please install Version "
-	  << Configuration.gtkmm_version.mymajor <<'.'
-	  << Configuration.gtkmm_version.myminor<<'.'
-	  << Configuration.gtkmm_version.mymicro<<"+))\n";
+	{  if (GTKMM2)
+	      f << "PKG_CHECK_MODULES(GTKMM,[gtkmm-2.0 >= "
+	      	<< Configuration.gtkmm_version << "])\n"
+	      	"AC_SUBST(GTKMM_CFLAGS)\n"
+	      	"AC_SUBST(GTKMM_LIBS)\n";
+	   else 
+	      f << "AM_PATH_GTKMM("<<Configuration.gtkmm_version <<",,\n"
+	           "\tAC_MSG_ERROR(Cannot find a matching GTK-- library:"
+	           " Please install version "
+	  	<< Configuration.gtkmm_version << " or newer))\n";
+	}
 	if (Configuration.gnome_support)
-	   f << "# GNOME-CONFIG script knows about gnomemm:\n"
+	{  if (GNOME2)
+	   {  f << "GNOMEMM_CFLAGS=\"`pkg-config --cflags libgnomemm-2.0 libgnomeuimm-2.0`\" \n"
+		   "GNOMEMM_LIBS=\"`pkg-config --libs libgnomemm-2.0 libgnomeuimm-2.0`\"\n"
+		   "AC_SUBST(GNOMEMM_CFLAGS)\n"
+		   "AC_SUBST(GNOMEMM_LIBS)\n";
+	      // FIXME: Need to detect if we need libbonobomm, libbonobouimm
+          }
+          else
+          {   f << "# GNOME-CONFIG script knows about gnomemm:\n"
 		   "# ('gnome-config' is installed by GNOME)\n"
-		   "GNOMEMM_CFLAGS=\"`$GNOME_CONFIG --cflags gnomemm`\" \n"
-		   "GNOMEMM_LIBS=\"`$GNOME_CONFIG --libs gnomemm`\"\n"
+		   "GNOMEMM_CFLAGS=\"`$GNOME_CONFIG --cflags gnomemm gnomeui`\" \n"
+		   "GNOMEMM_LIBS=\"`$GNOME_CONFIG --libs gnomemm gnomeui`\"\n"
 		   "AC_SUBST(GNOMEMM_CFLAGS)\n"
 		   "AC_SUBST(GNOMEMM_LIBS)\n";
 		// perhaps separate into GNOMEMM_LIBDIR and GNOMEMM_LIBS ?
 		// we should really test gnomemm!!!
+	   }
+	 }
+	if (Configuration.libglade_support && GTKMM2)
+	{  f << "PKG_CHECK_MODULES(LIBGLADE,[libglade-2.0 >= 2.0.0])\n"
+	      	"AC_SUBST(LIBGLADE_CFLAGS)\n"
+	      	"AC_SUBST(LIBGLADE_LIBS)\n";
+	}
 	 f << "\n";
 #if 0
 		"dnl Subst PACKAGE_PIXMAPS_DIR.\n"
@@ -451,18 +454,43 @@ void Cxx::WriteHeader(SystemFile &f,File_type tp)
 		"AC_SUBST(PACKAGE_PIXMAPS_DIR)\n"
 #endif
 	 f << "AC_OUTPUT(Makefile " << (Configuration.source_directory.c_str()+1) << "/Makefile ";
-	 if (Configuration.gettext_support) // && !Configuration.gnome_support)
-	 	// glade does this too ...
-	    f << "intl/Makefile ";
-	 if (Configuration.gettext_support)
-	    f << " po/Makefile.in ";
-	 if (Configuration.gnome_support)
-	    f << " macros/Makefile";
+	 if (!GTKMM2)
+	 {  if (Configuration.gettext_support && !Configuration.gnome_support)
+	 	// glade does this too ..., but does not work on debian
+	       f << "intl/Makefile ";
+	    if (Configuration.gettext_support)
+	       f << " po/Makefile.in ";
+	    if (Configuration.gnome_support)
+	       f << " macros/Makefile";
+	 }
 	 f << ")\n";
          break;
       case File_AUTOGEN_SH:
 	if (Configuration.gnome_support)
-	{  // this is a verbatim copy of gnome-skel 
+	{  if (GNOME2)
+	   {  f << "#!/bin/sh\n"
+		 "# Run this to generate all the initial makefiles, etc.\n"
+		 "\n"
+		 "srcdir=`dirname $0`\n"
+		 "test -z \"$srcdir\" && srcdir=.\n"
+		 "\n"
+		 "PKG_NAME=\"the package.\"\n"
+		 "\n"
+		 "(test -f $srcdir/configure.in) || {\n"
+		 "   echo -n \"**Error**: Directory \"\\`$srcdir\\'\" does not look like the\"\n"
+		 "   echo \" top-level directory\"\n"
+		 "   exit 1\n"
+		 "}\n"
+		 "\n"
+		 "which gnome-autogen.sh || {\n"
+		 "   echo \"You need to install gnome-common from the GNOME CVS\"\n"
+		 "   exit 1\n"
+		 "}\n"
+		 "USE_GNOME2_MACROS=1 . gnome-autogen.sh\n";
+           }
+           else
+           {
+	   // this is a verbatim copy of gnome-skel 
 	   f << "#!/bin/sh\n"
                 "# Run this to generate all the initial makefiles, etc.\n";
            WriteCreation(f,tp);
@@ -505,8 +533,10 @@ void Cxx::WriteHeader(SystemFile &f,File_type tp)
                 "    exit 1\n"
                 "}\n"
                 "\n"
-                ". $srcdir/macros/autogen.sh\n";
+                "export ACLOCAL_FLAGS=\"-I `pwd`/macros $ACLOCAL_FLAGS\"\n"
 //               "ACLOCAL_FLAGS=\"$ACLOCAL_FLAGS -I `aclocal --print-ac-dir`/gnome\" . `aclocal --print-ac-dir`/gnome/autogen.sh\n";
+                ". $srcdir/macros/autogen.sh\n";
+	   }
         }
 	else
 	{f << "#!/bin/sh\n";
@@ -535,8 +565,18 @@ void Cxx::WriteHeader(SystemFile &f,File_type tp)
 		"then echo you need automake to generate the Makefile\n"
 		"fi\n"
 		"\n"
-		"libtoolize --force --copy\n"
-		"autoheader\n";
+		"libtoolize --force --copy\n";
+	   if (Configuration.gettext_support)
+	      f << "gettextize\n"
+	      		// the following lines gave me an initial
+	      		// working gettext environment
+			"if test ! -e po/Makevars\n"
+			"then cp po/Makevars.template po/Makevars\n"
+			"fi\n"
+			"if test ! -e po/LINGUAS\n"
+			"then touch po/LINGUAS\n"
+			"fi\n";
+	   f << "autoheader\n";
 	   f << "aclocal\n";
 	   f << "automake --add-missing --copy --gnu\n"
 		"autoconf\n"
@@ -557,11 +597,13 @@ void Cxx::WriteHeader(SystemFile &f,File_type tp)
         }
         f << "\nSUBDIRS = "; 
         if (Configuration.gettext_support) 
-           f << "po "; // does gnome need intl? glade adds it ...
-        if (Configuration.gnome_support)
-           f << "macros ";
-        if (Configuration.gettext_support) 
-           f << "intl "; // does gnome really need intl?
+              f << "po "; // does gnome need intl? glade adds it ...
+        if (!GTKMM2)
+        {  if (Configuration.gnome_support)
+              f << "macros ";
+           if (!Configuration.gnome_support && Configuration.gettext_support) 
+              f << "intl "; // does gnome really need intl?
+        }
         f << (Configuration.source_directory.c_str()+1) << ' ';
         f << "\n"
           "\n"
@@ -577,6 +619,7 @@ void Cxx::WriteHeader(SystemFile &f,File_type tp)
         break;
       case File_AUTHORS:
       {
+#ifndef __MINGW32__      
         struct passwd *pwd=getpwuid(getuid());
         struct utsname uts;
         if (uname(&uts)) 
@@ -595,6 +638,7 @@ void Cxx::WriteHeader(SystemFile &f,File_type tp)
 #endif     
 #ifdef HAS_REAL_NAME
 	f << '>';
+#endif
 #endif
         f << '\n';
         break;
@@ -641,22 +685,64 @@ void Cxx::WriteFooter(CxxFile &f,File_type tp)
          f.StartBlock();
          if (Configuration.gettext_support)
          {  f.CppIf("defined(ENABLE_NLS)");
-            f.Statement("bindtextdomain (PACKAGE, PACKAGE_LOCALE_DIR)");
-            f.Statement("textdomain (PACKAGE)");
+            f.Statement("bindtextdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR)");
+            if (GTKMM2) 
+               f.Statement("bind_textdomain_codeset(GETTEXT_PACKAGE, \"UTF-8\")");
+            f.Statement("textdomain (GETTEXT_PACKAGE)");
             f.EndIf("ENABLE_NLS");
          }
 	 if (!Configuration.gnome_support)
 	    f.Declaration(WriterBase::GtkPrefix()+"Main m(&argc, &argv)");
-	 else
-	    f.Declaration() << WriterBase::GnomePrefix()+"Main m("
-	       << "PACKAGE, VERSION, argc, argv)";
+	 else { // perhaps trouble with gnome 2.0 and automake 1.5 ?
+	 	// because there PACKAGE is not defined ?
+	    if (GNOME2)
+	    {
+	       f.Declaration() << WriterBase::GnomePrefix()+"Main m("
+			       << "PACKAGE, VERSION, "
+			       << WriterBase::GnomeUIPrefix()+"module_info_get(), "
+			       << "argc, argv)";
+	       // FIXME: This is a kludge. Until the day libgnomeuimm
+	       // properly initializes bonobouimm (there's a FIXME in
+	       // the source there), we need to call wrap_init()
+	       // ourself.
+	       f.Statement("Gnome::Bonobo::wrap_init()");
+            }
+            else
+            {
+	       f.Declaration() << WriterBase::GnomePrefix()+"Main m("
+			       << "PACKAGE, VERSION, argc, argv)";
+	    }
+	 }
+	 // isn't there a possibility to set an image path?
 	 if (Configuration.use_libglade)
 	 {  std::string gnome(Configuration.gnome_support?"_gnome":"");
 	    f.Statement() << "glade"<<gnome<<"_init()";
 	 }
 	 f.EndLine();
          f.write_remembered(0);
-      	 f.Statement("m.run()").EndLine();
+      	 f.Statement("m.run(");
+      	 if (GTKMM2)
+      	 {  FOR_EACH_CONST_TAG(i,*top)
+      	    {  if (i->Type()=="widget")
+      	       {  Widget w(*i);
+      	          if (Configuration.libglade_support)
+      	          {  assert(w.getProperty("cxx_visibility","private")=="public");
+      	             // needs to access member widget pointer 
+      	             // since libglade classes do not inherit from Gtk::Widget
+      	             f << "*static_cast<" << 
+      	             // this is not correct, this should be a TypeName call
+      	             	Configuration.InstanceName(w.Name())
+      	               << "_glade*>(" << Configuration.InstanceName(w.Name())
+      	               << ")->" << Configuration.InstanceName(w.Name());
+      	          }
+      	          else
+      	             f << '*' << Configuration.InstanceName(w.Name());
+      	          break;
+      	       }
+      	    }
+      	 }
+      	 f <<")"; 
+      	 f.EndLine();
       	 f.write_remembered(1);     
       	 f.Statement("return 0");
       	 f.EndBlock();
@@ -681,21 +767,29 @@ void Cxx::WriteFooter(SystemFile &f,File_type tp)
       case File_MAKEFILE_AM:
          f << "\n\n";
       	 f.write_remembered();
-      	 f << "\n\n"
-      	      "CXXFLAGS = @CXXFLAGS@ ";
+      	 f << "\n\n";
+      	 if (Configuration.automake_version>=Pkg_Version(1,5,0)) 
+       	    f << "AM_CXXFLAGS = ";
+      	 else f << "CXXFLAGS = @CXXFLAGS@ ";
       	 if (Configuration.gnome_support)
-      	    f << "@GNOMEMM_CFLAGS@\n";
+      	    f << "@GNOMEMM_CFLAGS@ ";
       	 else
-      	    f << "@GTKMM_CFLAGS@\n";
-      	 if (Configuration.gnome_support)
+      	    f << "@GTKMM_CFLAGS@ ";
+      	 if (Configuration.libglade_support)
+      	    f << "@LIBGLADE_CFLAGS@ ";
+      	 f << '\n';
+      	 if (Configuration.gnome_support 
+      	 	&& Configuration.automake_version<Pkg_Version(1,5,0))
       	    f << "LDFLAGS= @LDFLAGS@ \n";
       	    
       	 f << '\n';
       	 f << AMName(Configuration.main_filename) << "_LDADD = @LIBS@ ";
       	 if (Configuration.gnome_support)
-      	    f << "@GNOMEMM_LIBS@";
+      	    f << "@GNOMEMM_LIBS@ ";
       	 else
-      	    f << "@GTKMM_LIBS@";
+      	    f << "@GTKMM_LIBS@ ";
+      	 if (Configuration.libglade_support)
+      	    f << "@LIBGLADE_LIBS@ ";
       	 f << '\n';
          break;
       case File_MAKEFILE:
@@ -725,6 +819,7 @@ void Cxx::WriteCreation(SystemFile &f,File_type tp)
    f.OStream().width(2);
    f << tm->tm_sec << ' ' << tzname[tm->tm_isdst];
    f.OStream().fill(oldfill); f.OStream().width(oldwidth);
+#ifndef __MINGW32__   
    struct passwd *pwd=getpwuid(getuid());
    struct utsname uts;
    if (uname(&uts)) 
@@ -737,8 +832,10 @@ void Cxx::WriteCreation(SystemFile &f,File_type tp)
      << uts.nodename 
 #ifdef HAS_DOMAINNAME     
      << '.' << uts.domainname 
-#endif     
-     << '\n';
+#endif
+	;
+#endif
+   f << '\n';
    f << comment << " using "GLADEMM_NAME"\n";
 }
 
