@@ -1,4 +1,4 @@
-// $Id: Cxx_Fileset.cc,v 1.65 2001/10/11 14:07:23 christof Exp $
+// $Id: Cxx_Fileset.cc,v 1.88 2003/06/18 10:35:48 christof Exp $
 /*  glade--: C++ frontend for glade (Gtk+ User Interface Builder)
  *  Copyright (C) 1998  Christof Petig
  *  Copyright (C) 1999-2000 Adolf Petig GmbH & Co. KG, written by Christof Petig
@@ -36,12 +36,22 @@ void Cxx_Fileset::WriteFiles(bool _toplevel)
    if (!Configuration.bare_bones) 
    { h.open(&w,File_FOO_HH); WriteHeader(h,File_FOO_HH);
      c.open(&w,File_FOO_CC); WriteHeader(c,File_FOO_CC);
+     if (Configuration.baseclass)
+     {  bc.open(&w,File_BASECLASSES_HH); WriteHeader(bc,File_BASECLASSES_HH); }
    
      if (_toplevel)
      {toplevel.main.Include(h.FileName(),true);
-      (Remember<CxxFile>(toplevel.main).Statement()
-      	  << "manage(new class " << Configuration.TypeName(w.Name()) << "())")
-      	 .EndLine();
+      Remember<CxxFile> os=Remember<CxxFile>(toplevel.main);
+       os.Declaration();
+       os << Configuration.TypeName(w.Name()) << " *";
+       os << Configuration.InstanceName(w.Name());
+       os.Assignment() << "new class " << Configuration.TypeName(w.Name()) 
+       	  << "()";
+       os.EndLine();
+      Remember<CxxFile> os2=Remember<CxxFile>(toplevel.main,1);
+       os2.Statement();
+       os2 << "delete " << Configuration.InstanceName(w.Name());
+       os2.EndLine();
      }
 
       if (!Configuration.no_autoconf)
@@ -62,9 +72,11 @@ void Cxx_Fileset::WriteFiles(bool _toplevel)
    
    if (!Configuration.bare_bones) 
    {  c.Include(h.FileName(),true);
-      gc.Include(h.FileName(),true);
+//      gc.Include(h.FileName(),true);
    }
-   gc.Include(gc.FileName(File_FOO_HH),true);
+   if (Configuration.non_virtual_callbacks || Configuration.need_notebook_hack) 
+      gc.Include(h.FileName(),true);
+   else gc.Include(gh.FileName(),true);
 
    WriteClasses(w);
    
@@ -74,6 +86,8 @@ void Cxx_Fileset::WriteFiles(bool _toplevel)
    }
    WriteFooter(gc,File_FOO_GLADE_CC); gc.close();
    WriteFooter(gh,File_FOO_GLADE_HH); gh.close();
+   if (Configuration.baseclass)
+   {  WriteFooter(bc,File_BASECLASSES_HH); bc.close(); }
 }
 
 void Cxx_Fileset::DoIncludes(const Widget &w)
@@ -97,12 +111,19 @@ void Cxx_Fileset::WriteClasses(const Widget &w)
    // Include own type and type of all included widgets
 //   std::cerr << "WriteClasses(" << w.Name() << ") {\n";
    DoIncludes(w);
+   if (Configuration.baseclass)
+   {  gh.Include("./"+Configuration.FileName("",File_BASECLASSES_HH,File_NODIR),true);
+   }
    // start class generation
 
    /******* GH: class Xyz_Glade ******/
+   if (Configuration.debug) 
+      std::cout << "======== GH declaration ============\n";
    const WriterBase &top_wr(LookupWriter(w));
    gh.Definition().Class(Configuration.TypeName(w.Name(),true));
    top_wr.Derivation(w,gh);
+   if (Configuration.baseclass) 
+      gh.Derive("public "+Configuration.TypeName(w.Name())+"_base");
    gh.StartBlock();
    top_wr.AdditionalMemberVars(w,gh,true);
 
@@ -118,7 +139,7 @@ void Cxx_Fileset::WriteClasses(const Widget &w)
    gh.Protected().Declaration()
    	.FunctionName(Configuration.TypeName(w.Name(),true));
    if (Configuration.has_accelerators && top_wr.needDataArg(w))
-   {  gh.FunctionArg("GlademmData *_data");
+   {  gh.FunctionArg("GlademmData *gmm_data");
    }
 #ifdef NEED_CONSTRUCT_ARGS
    top_wr.ConstructArgs(w,gh);
@@ -132,31 +153,52 @@ void Cxx_Fileset::WriteClasses(const Widget &w)
    gh.EndLine();
    gh.Declaration()
         .FunctionName("~"+Configuration.TypeName(w.Name(),true)).EndLine();
+   // virtual signal handlers
+   if (!Configuration.non_virtual_callbacks)
+   {  for (Widget::const_contained_iterator i=w.begin_contained(Internal_Both);
+   		i!=w.end_contained();++i)
+      {  Widget w2(*i);
+         const WriterBase &wr(LookupWriter(w2,true));
+         DeclareSignalHandler(wr,w2,w,false,true);
+      }
+      DeclareSignalHandler(top_wr,w,w,true,true);
+   }
    gh.EndBlock();
    
    /******* GH: defines (glademm_get) ***********/
-   if (w.getString("cxx_visibility","private")=="public") 
-   // we can't use top_wr since we need the component type, not the widget type
-      GenerateMapDefine(LookupWriter(w,true),w);
-   for (Widget::const_contained_iterator i=w.begin_contained();
-	i!=w.end_contained();++i)
-   {  Widget w2(*i);
-      if (w2.getString("cxx_visibility","private")=="public"
-         	 && !w2.getBool(CXX_SEPERATE_CLASS)) 
-         GenerateMapDefine(LookupWriter(w2,true),w2);
+   if (Configuration.lookup_table)
+   {  if (w.getProperty("cxx_visibility","private")=="public") 
+      // we can't use top_wr since we need the component type, not the widget type
+         GenerateMapDefine(LookupWriter(w,true),w);
+      for (Widget::const_contained_iterator i=w.begin_contained();
+   	i!=w.end_contained();++i)
+      {  Widget w2(*i);
+         if (w2.getProperty("cxx_visibility","private")=="public"
+            	 && !w2.getBoolProperty(CXX_SEPERATE_CLASS)) 
+            GenerateMapDefine(LookupWriter(w2,true),w2);
+      }
    }
    
    /******* H: class Xyz ******/
+   if (Configuration.baseclass)
+   {  bc.Definition().Class(Configuration.TypeName(w.Name())+"_base");
+      bc.StartBlock();
+      bc.EndBlock();
+   }
+   if (Configuration.debug) 
+      std::cout << "======== H declaration ============\n";
    h.Definition().Class(Configuration.TypeName(w.Name()));
    h.Derive("public "+Configuration.TypeName(w.Name(),true));
    h.StartBlock();
 
-   h.Declaration() << "friend class " << Configuration.TypeName(w.Name(),true);
-   h.EndLine();
+   if (Configuration.non_virtual_callbacks || Configuration.need_notebook_hack)
+   {  h.Declaration() << "friend class " << Configuration.TypeName(w.Name(),true);
+      h.EndLine();
+   }
    if (Configuration.has_accelerators && top_wr.needDataArg(w))
    {  h.Public().Definition().FunctionName(Configuration.TypeName(w.Name()))
-   	.FunctionArg("GlademmData *_data")
-   	.Construct(Configuration.TypeName(w.Name(),true),"_data")
+   	.FunctionArg("GlademmData *gmm_data")
+   	.Construct(Configuration.TypeName(w.Name(),true),"gmm_data")
    	.StartBlock().EndBlock();
    } 
    for (Widget::const_contained_iterator i=w.begin_contained(Internal_Both);
@@ -171,6 +213,8 @@ void Cxx_Fileset::WriteClasses(const Widget &w)
    
    /****** GC: include, definitions **************/
    
+   if (Configuration.debug) 
+      std::cout << "======== GC include ============\n";
    top_wr.GCInclude(w,gc);
    top_wr.GCDefinition(w,gc);
    for (Widget::const_contained_iterator i=w.begin_contained(Internal_Both);
@@ -182,11 +226,13 @@ void Cxx_Fileset::WriteClasses(const Widget &w)
    }
    
    /****** GC: ctor **************/
+   if (Configuration.debug) 
+      std::cout << "======== GC ctor ============\n";
    ClearRadioGroups();
    gc.Definition().FunctionName() << Configuration.TypeName(w.Name(),true)
    	<< "::" << Configuration.TypeName(w.Name(),true);
    if (Configuration.has_accelerators && top_wr.needDataArg(w))
-   {  gc.FunctionArg("GlademmData *_data");
+   {  gc.FunctionArg("GlademmData *gmm_data");
    }
 #ifdef NEED_CONSTRUCT_ARGS
    top_wr.ConstructArgs(w,gc);
@@ -209,9 +255,12 @@ void Cxx_Fileset::WriteClasses(const Widget &w)
    if (top_wr.CantMemberConstruct(w))
    {  std::cerr << "ERROR: " << w.Name() << ": Can't form a class of its own (yet).\n";
    }
+   // is this order intended? T.L. suggested to change it because of menubar.cc
    top_wr.CreatePointer_Toplevel(w,gc);
    top_wr.ClassConstructor(w,gc);
-   
+
+   if (Configuration.debug) 
+      std::cout << "======== creating the widgets ============\n";
    // create widgets
    for (Widget::const_contained_iterator i=w.begin_contained(NoInternal,Configuration.debug);
 	i!=w.end_contained();++i)
@@ -220,58 +269,83 @@ void Cxx_Fileset::WriteClasses(const Widget &w)
       wr.CreatePointer(w2,gc);
    }
    
+   top_wr.CreatePointer_depending(w,gc);
+   for (Widget::const_contained_iterator i=w.begin_contained(NoInternal,Configuration.debug);
+	i!=w.end_contained();++i)
+   {  Widget w2(*i);
+      const WriterBase &wr(LookupWriter(w2,true));
+      wr.CreatePointer_depending(w2,gc);
+   }
+
+   if (Configuration.debug) 
+      std::cout << "======== configuring the widgets ============\n";
    // configure widgets
    for (Widget::const_contained_iterator i=w.begin_contained(Internal_Both,Configuration.debug);
 	i!=w.end_contained();++i)
-   {  if (!i->getBool(CXX_SEPERATE_CLASS))
-      {  const Widget WriterWidget(*i.getWriterTag());
-         Widget w2(*i);
-         const WriterBase &ww_wr(LookupWriter(WriterWidget));
-         std::string instance(ww_wr.Instance(WriterWidget,w2,i.getSWType()));
-         if (!instance.size()) 
-         {  std::cerr << "can't modify " << WriterWidget.Name() << ':' << w2.Name() << "\n";
+   {  const Widget w2=*i;
+      if (!w2.getBoolProperty(CXX_SEPERATE_CLASS))
+      {  const Widget WriterWidget=*i.getWriterTag();
+         const WriterBase &ww_wr=LookupWriter(WriterWidget);
+         std::string instance=ww_wr.Instance(WriterWidget,w2,i.getSWType());
+         if (instance.empty()) 
+         {  if (Configuration.verbose>1)
+               std::cerr << "Warning: can't change properties of " 
+               		<< WriterWidget.Name() << ':' << w2.Name() << "\n";
             continue; // can't modify this widget
          }
-         const WriterBase &wr2(LookupWriter(w2,true));
+         const WriterBase &wr2=LookupWriter(w2,true);
          wr2.Configure(w2,gc,instance);
-         wr2.AddChildren(w2,gc,instance);
+        // since we go down one step:
+	// determine whether w2's children might be subwidgets of
+	// i's parent
+	 if (i.getSWType()!=is_Subwidget && i.getSWType()!=is_Subwidget_all) 
+	 // note that is_Subwidget_only is ok
+	 {  wr2.AddChildren(w2,gc,instance,wr2,w2);
+	 }
+	 else
+	 {  wr2.AddChildren(w2,gc,instance,ww_wr,WriterWidget);
+	 }
 //         wr2.Constructor(w2,gc,instance);
       }
    }
-   const std::string inst(top_wr.Instance(w,w,not_Subwidget));
+   const std::string inst=top_wr.Instance(w,w,not_Subwidget);
    top_wr.Configure(w,gc,inst);
-   top_wr.AddChildren(w,gc,inst);
+   top_wr.AddChildren(w,gc,inst,top_wr,w);
 //   top_wr.Constructor(w,gc,inst);
    
+   if (Configuration.debug) 
+      std::cout << "======== showing the widgets ============\n";
    // show widgets (no need to show internal ones)
    for (Widget::const_contained_iterator i=w.begin_contained();
 	i!=w.end_contained();++i)
    {  Widget w2(*i);
-      if (!w2.getBool(CXX_SEPERATE_CLASS))
+      if (!w2.getBoolProperty(CXX_SEPERATE_CLASS))
       {  const Widget WriterWidget(*i.getWriterTag());
          const WriterBase &ww_wr(LookupWriter(WriterWidget));
          std::string instance(ww_wr.Instance(WriterWidget,w2,i.getSWType()));
-         if (!instance.size()) 
+         if (instance.empty()) 
          {  std::cerr << "Configure_show has problems with " 
          	<< WriterWidget.Name() << ':' << w2.Name() << '\n';
             continue;
          }
-//         assert(instance.size()); // we should not see internal widgets!
+//         assert(!instance.empty()); // we should not see internal widgets!
          const WriterBase &wr(LookupWriter(w2,true));
          wr.Configure_show(w2,gc,instance);
       }
    }
    top_wr.Configure_show(w,gc,inst);
    
+   if (Configuration.debug) 
+      std::cout << "======== connecting callbacks ============\n";
    // Connect callbacks
    for (Widget::const_contained_iterator i=w.begin_contained(Internal_Both);
 	i!=w.end_contained();++i)
-   {  if (!i->getBool(CXX_SEPERATE_CLASS))
+   {  Widget w2(*i);
+      if (!w2.getBoolProperty(CXX_SEPERATE_CLASS))
       {  const Widget WriterWidget(*(i.getWriterTag()));
-         Widget w2(*i);
          const WriterBase &ww_wr(LookupWriter(WriterWidget));
          std::string instance(ww_wr.Instance(WriterWidget,w2,i.getSWType()));
-         if (!instance.size()) 
+         if (instance.empty()) 
          {  continue; // can't connect this widget
          }
          const WriterBase &wr2(LookupWriter(w2,true));
@@ -281,20 +355,24 @@ void Cxx_Fileset::WriteClasses(const Widget &w)
    ConnectSignalHandler(top_wr,w,w,inst);
 
    // add them to the map
-   if (w.getString("cxx_visibility","private")=="public") 
-   // we can't use top_wr since we need the component type, not the widget type
-      AddToMap(w,"this");
-   for (Widget::const_contained_iterator i=w.begin_contained();
-	i!=w.end_contained();++i)
-   {  Widget w2(*i);
-      if (w2.getString("cxx_visibility","private")=="public"
-         	 && !w2.getBool(CXX_SEPERATE_CLASS)) 
-         AddToMap(w2,WriterBase::Pointer(w2));
+   if (Configuration.lookup_table)
+   {  if (w.getProperty("cxx_visibility","private")=="public") 
+      // we can't use top_wr since we need the component type, not the widget type
+         AddToMap(w,"this");
+      for (Widget::const_contained_iterator i=w.begin_contained();
+   	i!=w.end_contained();++i)
+      {  Widget w2(*i);
+         if (w2.getProperty("cxx_visibility","private")=="public"
+            	 && !w2.getBoolProperty(CXX_SEPERATE_CLASS)) 
+            AddToMap(w2,WriterBase::Pointer(w2));
+      }
    }
    
    gc.EndBlock();
 
    /************ GC: dtor *************/
+   if (Configuration.debug) 
+      std::cout << "======== GC dtor ============\n";
    gc.Definition().FunctionName() << Configuration.TypeName(w.Name(),true)
    	<< "::~" << Configuration.TypeName(w.Name(),true);
    gc.StartBlock();
@@ -312,10 +390,12 @@ void Cxx_Fileset::WriteClasses(const Widget &w)
    
    /****** C: signal handler stubs *******/
    
+   if (Configuration.debug) 
+      std::cout << "======== C signal stubs ============\n";
    for (Widget::const_contained_iterator i=w.begin_contained(Internal_Both);
 	i!=w.end_contained();++i)
    {  Widget w2(*i);
-      if (w2.getBool(CXX_SEPERATE_CLASS)) continue;
+      if (w2.getBoolProperty(CXX_SEPERATE_CLASS)) continue;
       const WriterBase &wr(LookupWriter(w2,true));
       DefineSignalHandler(wr,w2,w);
    }
@@ -324,14 +404,16 @@ void Cxx_Fileset::WriteClasses(const Widget &w)
    
    /****** other classes/files ********/
    // recurse
+   if (Configuration.debug) 
+      std::cout << "======== separate-file widgets ============\n";
    for (Widget::const_contained_iterator i=w.begin_contained();
 	i!=w.end_contained();++i)
    {  Widget w2(*i);
-      if (w2.getBool(CXX_SEPERATE_FILE))
+      if (w2.getBoolProperty(CXX_SEPERATE_FILE))
       {  Cxx_Fileset sub_fs(*(w2.getTagPtr()),toplevel);
          sub_fs.WriteFiles();
       }
-      else if (w2.getBool(CXX_SEPERATE_CLASS))
+      else if (w2.getBoolProperty(CXX_SEPERATE_CLASS))
          WriteClasses(w2);
    }
 //   std::cerr << "}\n";
@@ -362,12 +444,8 @@ void Cxx_Fileset::WriteHeader(WidgetFile &f,File_type tp)
          Cxx::WriteCreation(f,tp);
       	 f << "//\n// DO NOT EDIT THIS FILE ! It was created using\n// " 
       	   << Configuration.commandline << "\n"
-      	      "// for gtk " <<Configuration.gtk_version.mymajor <<'.'
-			<< Configuration.gtk_version.myminor<<'.'
-			<< Configuration.gtk_version.mymicro<<
-	      " and gtkmm " <<Configuration.gtkmm_version.mymajor <<'.'
-			<< Configuration.gtkmm_version.myminor<<'.'
-			<< Configuration.gtkmm_version.mymicro<< "\n"
+      	      "// for gtk " <<Configuration.gtk_version <<
+	      " and gtkmm " <<Configuration.gtkmm_version << "\n"
       	      "//\n// Please modify the corresponding derived classes in ";
       	 f << Configuration.FileName(f.widgetName(),File_FOO_HH) << " and"
       	   << Configuration.FileName(f.widgetName(),File_FOO_CC) << "\n"
@@ -375,40 +453,78 @@ void Cxx_Fileset::WriteHeader(WidgetFile &f,File_type tp)
       	      "\n#ifndef "<<Configuration.FileDefine(f.widgetName(),f.type())<<"\n" 
       	      "#  define "<<Configuration.FileDefine(f.widgetName(),f.type())<<"\n\n";
       	 if (Configuration.has_accelerators)
-      	 {  f.CppIf() << "!defined(GLADEMM_DATA)";
+      	 {  std::string agtype=WriterBase::GtkPrefix()+"AccelGroup *";
+      	    if (GTKMM2) agtype="Glib::RefPtr<Gtk::AccelGroup> ";
+      	 
+      	    f.CppIf() << "!defined(GLADEMM_DATA)";
       	    f.DefineName("GLADEMM_DATA").DefineBody().EndLine();
-      	    f.Include("gtk--/accelgroup.h");
+      	    f.Include(Configuration.GtkmmIncludePath()+"accelgroup.h");
       	    f.Definition().Class("GlademmData").StartBlock();
-      	    f.Declaration(WriterBase::GtkPrefix()+"AccelGroup *accgrp");
+      	    f.Declaration(agtype+"accgrp");
       	    f.Public().Definition().FunctionName("GlademmData")
-      	     	.FunctionArg(WriterBase::GtkPrefix()+"AccelGroup *ag").Construct("accgrp","ag")
+      	     	.FunctionArg(agtype+"ag").Construct("accgrp","ag")
       	     	.StartBlock().EndBlock();
-      	    f.Definition().Funct_ReturnType(WriterBase::GtkPrefix()+"AccelGroup *")
+      	    f.Definition().Funct_ReturnType(agtype)
       	    	.FunctionName("getAccelGroup")
       	    	.StartBlock().Statement("return accgrp").EndBlock();
       	    f.EndBlock();
       	    f.EndIf() << "GLADEMM_DATA";
       	    f.EndLine(); // needed for sanity
       	 }
+      	 if (Configuration.use_libglade)
+      	 {  f.Include("glade/glade.h");
+      	 }
       	 break;
       case File_FOO_GLADE_CC:
          Cxx::WriteCreation(f,tp);
       	 f << "//\n// DO NOT EDIT THIS FILE ! It was created using\n// " 
       	   << Configuration.commandline << "\n"
-      	      "// for gtk " <<Configuration.gtk_version.mymajor <<'.'
-			<< Configuration.gtk_version.myminor<<'.'
-			<< Configuration.gtk_version.mymicro<<
-	      " and gtkmm " <<Configuration.gtkmm_version.mymajor <<'.'
-			<< Configuration.gtkmm_version.myminor<<'.'
-			<< Configuration.gtkmm_version.mymicro<< "\n"
+      	      "// for gtk " <<Configuration.gtk_version<<
+	      " and gtkmm " <<Configuration.gtkmm_version<< "\n"
       	      "//\n// Please modify the corresponding derived classes in ";
       	 f << Configuration.FileName(f.widgetName(),File_FOO_CC) << "\n"
       	 	"\n";
+      	 
+      	 if (GTKMM2)
+      	 {  f.CppIf() << "defined __GNUC__ && __GNUC__ < 3";
+      	    f << "\n"
+      	      << "#error This program will crash if compiled with g++ 2.x\n"
+      	      << "// see the dynamic_cast bug in the gtkmm FAQ";
+      	    f.EndIf();
+      	 }
+      	 f.Include("config.h",true); // always a good idea !!!
+
       	 // TODO: we might include only the libgnome includes we need
       	 // to speed up compilation
-      	 f.Include("config.h",true); // always a good idea !!!
 	 if (Configuration.gettext_support)
-	    f.Include("libgnome/libgnome.h"); // or gnome-i18n.h ?
+	 {  if (Configuration.gnome_support)
+	       f.Include("libgnome/libgnome.h"); // or gnome-i18n.h ?
+	    else
+	    {  // like glade does it
+	       f.EndLine();
+               f << "/*\n"
+                  " * Standard gettext macros.\n"
+                  " */\n"
+                  "#ifdef ENABLE_NLS\n"
+                  "#  include <libintl.h>\n"
+                  "#  undef _\n"
+                  "#  define _(String) dgettext (GETTEXT_PACKAGE, String)\n"
+                  "#  ifdef gettext_noop\n"
+                  "#    define N_(String) gettext_noop (String)\n"
+                  "#  else\n"
+                  "#    define N_(String) (String)\n"
+                  "#  endif\n"
+                  "#else\n"
+                  "#  define textdomain(String) (String)\n"
+                  "#  define gettext(String) (String)\n"
+                  "#  define dgettext(Domain,Message) (Message)\n"
+                  "#  define dcgettext(Domain,Message,Type) (Message)\n"
+                  "#  define bindtextdomain(Domain,Directory) (Domain)\n"
+                  "#  define _(String) (String)\n"
+                  "#  define N_(String) (String)\n"
+                  "#endif\n";
+	    }
+	 }
          break;
       case File_FOO_CC:
          Cxx::WriteCreation(f,tp);
@@ -422,6 +538,15 @@ void Cxx_Fileset::WriteHeader(WidgetFile &f,File_type tp)
       	    f << '\n';
       	 }
    	 break;
+      case File_BASECLASSES_HH:
+         Cxx::WriteCreation(f,tp);
+   	 f << "//\n// newer (non customized) versions of this file go to ";
+   	 f << Configuration.FileName("",tp,File_GLADE|File_NODIR) << "\n"
+   	 	"\n"
+		"// This file is for your program, I won't touch it again!\n\n";
+	 f.CppIf("!defined("+Configuration.FileDefine("",tp)+")");
+	 f.DefineName(Configuration.FileDefine("",tp)).DefineBody().EndLine();
+   	 break;
       default: assert(0);
    }	
 }
@@ -434,6 +559,9 @@ void Cxx_Fileset::WriteFooter(WidgetFile &f,File_type tp)
       case File_FOO_GLADE_HH:
    	 f << "#endif\n";
    	 break;
+      case File_BASECLASSES_HH:
+         f.EndIf();
+         break;
       default: 
          break;
    }
