@@ -1,4 +1,4 @@
-// $Id: Naming.cc,v 1.26 2001/10/11 14:07:23 christof Exp $
+// $Id: Naming.cc,v 1.37 2003/04/25 07:23:32 christof Exp $
 /*  glade--: C++ frontend for glade (Gtk+ User Interface Builder)
  *  Copyright (C) 1998  Christof Petig
  *  Copyright (C) 1999-2000 Adolf Petig GmbH & Co. KG, written by Christof Petig
@@ -68,6 +68,7 @@ const std::string Naming::FileName(const std::string &tag,File_type tp,int flags
          case File_SUPPORT_HH:
          case File_SUPPORT_CC:
          case File_MAKEFILE_AM:
+         case File_BASECLASSES_HH:
             ret=Configuration.destination_directory
             	+Configuration.source_directory+"/";
             break;
@@ -158,6 +159,11 @@ const std::string Naming::FileName(const std::string &tag,File_type tp,int flags
       case File_NEWS: ret+="NEWS"; break;
       case File_TODO: ret+="TODO"; break;
       case File_po_POTFILES_in: ret+="POTFILES.in"; break;
+      case File_BASECLASSES_HH:
+         ret+=Configuration.main_filename;
+      	 ret+="_base";
+         ret+= Configuration.header_suffix;
+         break;
       default:
          std::cerr<<__FILELINE__" attempt to create unknown file type " << int(tp) << "\n";
          ret+='?';
@@ -177,6 +183,7 @@ const std::string Naming::FileName(const std::string &tag,File_type tp,int flags
       case File_CONFIGURE_IN:
       case File_AUTOGEN_SH:
       case File_MAIN_CC:
+      case File_BASECLASSES_HH:
          if ((flags&File_GLADE) || ((flags&File_NOREPLACE) && !access(ret.c_str(),F_OK)))
          {  ret+=Configuration.template_postfix;
          }
@@ -203,6 +210,9 @@ const std::string Naming::FileDefine(const std::string &tag,File_type tp) const
       case File_FOO_HH:
          ret+= Configuration.header_suffix;
          break;
+      case File_BASECLASSES_HH:
+         ret+= Configuration.main_filename+"baseclasses"+Configuration.header_suffix;
+         break;
       default:
          std::cerr << __FILELINE__" attempt to create unknown define type " << int(tp) << "\n";
          break;
@@ -227,18 +237,28 @@ const std::string Naming::CName(const std::string &tag) const throw()
 {  return ToCIdentifier(tag);
 }
 
+// since people seem to use identifiers like '1', translate them to 
+// something different from '_'
+
+static const char * const digitnames[10] 
+= { "zero", "one", "two", "three", "four", "five", "six", "seven", "eight", "nine" };
+
 const std::string Naming::ToCIdentifier(const std::string &tag) throw()
 {  std::string ret("");
    std::string::const_iterator i=tag.begin();
    // first char [_a-zA-Z]
    if (i!=tag.end()) 
-   {  if (!(*i=='_' || isalpha((unsigned char)*i))) ret+='_';
+   {  if ('0' <= *i&&*i <= '9') ret+=digitnames[*i-'0'];
+      else if (!(*i=='_' || ('a' <= *i&&*i <= 'z') || ('A' <= *i&&*i <= 'Z'))) 
+      	 ret+='_';
       else ret+=*i;
       ++i;
    }
    // now [_a-zA-Z0-9]
    for (;i!=tag.end();++i) 
-   {  if (!(*i=='_' || isalnum((unsigned char)*i))) ret+='_';
+   {  if (!(*i=='_' || ('a' <= *i&&*i <= 'z') || ('A' <= *i&&*i <= 'Z') 
+   		|| ('0' <= *i&&*i <= '9'))) 
+      	 ret+='_';
       else ret+=*i;
    }
    return ret;
@@ -276,7 +296,7 @@ const std::string Naming::ToFileName(const std::string &x) throw()
    for (;i!=x.end();++i)
    {  switch((unsigned char)*i)
       {  case 0 ... 31:
-         case 127 ... 159:
+//         case 127 ... 159:  these are valid UTF8 !
       	    ret+="_";
             break;
          default: ret+=*i;
@@ -286,3 +306,54 @@ const std::string Naming::ToFileName(const std::string &x) throw()
    return ret;
 }
 
+const std::string Configuration::GtkmmIncludePath() const throw()
+{  if (Gtkmm2()) return "gtkmm/";
+   return "gtk--/";
+}
+
+const std::string Configuration::GnomeUImmIncludePath() const throw()
+{  return Gnome2() ? "libgnomeuimm/" : "gnome--/";
+}
+
+const std::string Configuration::GnomemmIncludePath() const throw()
+{  return Gnome2() ? "libgnomemm/" : "gnome--/";
+}
+
+const std::string Configuration::BonobommIncludePath() const throw()
+{  return "bonobomm/";
+}
+
+const std::string Configuration::BonoboUImmIncludePath() const throw()
+{  return "bonobomm/";
+}
+
+void Configuration::select_Gnome1()
+{  if (!gnome2) return;
+      gnome2=false;
+      gtk_version.set_if_more_specific(Pkg_Version(GTK_MAJOR,GTK_MINOR,GTK_MICRO,Pkg_Version::Glademm_Compile_Time));
+      gtkmm_version.set_if_more_specific(Pkg_Version(GTKMM_MAJOR,GTKMM_MINOR,GTKMM_MICRO,Pkg_Version::Glademm_Compile_Time));
+#ifdef GNOMEMM_MAJOR
+      libgnomemm_version.set_if_more_specific(Pkg_Version(GNOMEMM_MAJOR,GNOMEMM_MINOR,GNOMEMM_MICRO,Pkg_Version::Glademm_Compile_Time));
+      libgnomeuimm_version.set_if_more_specific(Pkg_Version(GNOMEMM_MAJOR,GNOMEMM_MINOR,GNOMEMM_MICRO,Pkg_Version::Glademm_Compile_Time));
+#endif
+}
+
+void Configuration::select_Gnome2()
+{  if (gnome2) return;
+      gnome2=true;
+      gtk_version.set_if_more_specific(Pkg_Version(GTK2_MAJOR,GTK2_MINOR,GTK2_MICRO,Pkg_Version::Glademm_Compile_Time));
+      gtkmm_version.set_if_more_specific(Pkg_Version(GTKMM2_MAJOR,GTKMM2_MINOR,GTKMM2_MICRO,Pkg_Version::Glademm_Compile_Time));
+#ifdef LIBGNOMEMM_MAJOR
+      libgnomemm_version.set_if_more_specific(Pkg_Version(LIBGNOMEMM_MAJOR,LIBGNOMEMM_MINOR,LIBGNOMEMM_MICRO,Pkg_Version::Glademm_Compile_Time));
+#endif
+#ifdef LIBGNOMEUIMM_MAJOR
+      libgnomeuimm_version.set_if_more_specific(Pkg_Version(LIBGNOMEUIMM_MAJOR,LIBGNOMEUIMM_MINOR,LIBGNOMEUIMM_MICRO,Pkg_Version::Glademm_Compile_Time));
+#endif
+#ifdef LIBBONOBOMM_MAJOR
+      libbonobomm_version.set_if_more_specific(Pkg_Version(LIBBONOBOMM_MAJOR,LIBBONOBOMM_MINOR,LIBBONOBOMM_MICRO,Pkg_Version::Glademm_Compile_Time));
+#endif
+#ifdef LIBBONOBOUIMM_MAJOR
+      libbonobouimm_version.set_if_more_specific(Pkg_Version(LIBBONOBOUIMM_MAJOR,LIBBONOBOUIMM_MINOR,LIBBONOBOUIMM_MICRO,Pkg_Version::Glademm_Compile_Time));
+#endif
+      has_accelerators=true; // make it standard
+}
