@@ -1,4 +1,4 @@
-// $Id: CxxFile.cc,v 1.24 2002/03/01 21:50:24 christof Exp $
+// $Id: CxxFile.cc,v 1.28 2003/05/19 08:51:25 christof Exp $
 /*  glade--: C++ frontend for glade (Gtk+ User Interface Builder)
  *  Copyright (C) 1998  Christof Petig
  *  Copyright (C) 1999-2000 Adolf Petig GmbH & Co. KG, written by Christof Petig
@@ -23,7 +23,9 @@
 
 #include "CxxFile.hh"
 #include <cstdio>
-#include <algo.h>
+#include <algorithm>
+#include <cassert>
+#include <iostream>
 
 CxxFile &CxxFile::GlobalContext(Global_Context new_gc,bool stay_inside)
 // insert blank line if necessary
@@ -43,7 +45,7 @@ void CxxFile::close(void)
 {  EndLine(); // necessary??
    ToOutside();
    SystemFile::close();
-   includes.erase(includes.begin(),includes.end());
+   elements.clear();
 }
 
 CxxFile &CxxFile::ToOutside()
@@ -193,18 +195,17 @@ CxxFile &CxxFile::EndLine(bool endBlock)
 }
 
 CxxFile &CxxFile::Include(const std::string &name,bool local)
-{  if (!name[0]) return *this; // suchen ob schon ? 
+{  if (name.empty()) return *this; // suchen ob schon ? 
 
-   const std::string extname((local?'"':'<')+name);
-   if (find(includes.begin(),includes.end(),extname)!=includes.end()) 
-   	return *this;
+   const UniqueValue::value_t type=local?v_IncludeQuote:v_IncludeBracket;
+   if (ElementAlreadyThere(element_t(type,name))) return *this;
    	
 //   ToOutside();
    GlobalContext(gc_include); // insert blank lines if necessary
-   Output() << "#include " << extname << (local?'"':'>');
+   Output() << "#include " << (local?'"':'<') << name << (local?'"':'>');
    NewLine(false);
    // FIXME: cpp indentation
-   includes.push_back(extname);
+   AddElement(element_t(type,name));
    return *this;
 }
 
@@ -503,7 +504,7 @@ void CxxFile::new_context(CxxFile_Context c)
    }
 }
 
-void CxxFile::push(CxxFile_Context c,int indent_delta=0)
+void CxxFile::push(CxxFile_Context c,int indent_delta)
 {  pushed.push_back(state);
 #ifdef DEBUG_STATE_MACHINE
    std::cout << '+' << state.context << "->" << c << " | " << state.global << '\n';
@@ -644,3 +645,23 @@ SystemFile &CxxFile::Output()
    empty_line=false;
    return *this;
 }
+
+CxxFile &CxxFile::FunctionEndArgs()
+{  switch (state.context)
+   {  case ctx_FunctionName: // no function args
+      	 Output() << "()";
+      	 pop();
+      	 break;
+      case ctx_MidFunctionArgs: // end function
+         Output() << ')';
+         pop();
+         break;
+      default: InvalidState("FunctionEndArgs"); break;
+   }
+   return *this;
+}
+
+UniqueValue CxxFile::element_types;
+const UniqueValue::value_t CxxFile::v_IncludeBracket=element_types.get();
+const UniqueValue::value_t CxxFile::v_IncludeQuote=element_types.get();
+
