@@ -1,4 +1,4 @@
-// $Id: adjustment.cc,v 1.15 2001/10/11 14:07:24 christof Exp $
+// $Id: adjustment.cc,v 1.18 2002/12/19 08:55:54 christof Exp $
 /*  glade--: C++ frontend for glade (Gtk+ User Interface Builder)
  *  Copyright (C) 1998  Christof Petig
  *  Copyright (C) 1999-2000  Adolf Petig GmbH & Co. KG, written by Christof Petig
@@ -19,6 +19,7 @@
  */
 
 #include "adjustment.hh"
+#include "stringtok.h"
 
 static Gtk_Adjustment Gtk_Adjustment;
 //static Gtk_Adjustment hGtk_Adjustment('h');
@@ -29,7 +30,7 @@ const std::string Gtk_Adjustment::TypeName(const Widget &w) const
 }
 
 const std::string Gtk_Adjustment::IncludeName(const Widget &w) const
-{  return "gtk--/adjustment.h";
+{  return Configuration.GtkmmIncludePath()+"adjustment.h";
 }
 
 Gtk_Adjustment::Gtk_Adjustment()
@@ -37,17 +38,27 @@ Gtk_Adjustment::Gtk_Adjustment()
 }
 
 void Gtk_Adjustment::ConstructionArgs(const Widget &w, CxxFile &f) const
-{  float value(w.getFloat("value",0));
-   float lower(w.getFloat("lower",0));
-   float upper(w.getFloat("upper",100));
-   float step(w.getFloat("step",1));
-   float page(w.getFloat("page",10));
-   float page_size(w.getFloat("page_size",10));
-      
+{  float value=0,lower=0,upper=100,step=1,page=10,page_size=10;
+   if (w.hasProperty("adjustment"))
+   {  std::list<std::string>  list;
+      stringtok (list, w.getProperty("adjustment"));
+      for (std::list<std::string>::iterator i = list.begin(); i != list.end(); ++i)
+      {  f.FunctionArg() << (*i);
+      }
+   }
+   else
+   {  value=w.getFloatProperty("value",value);
+      lower=w.getFloatProperty("lower",lower);
+      upper=w.getFloatProperty("upper",upper);
+      step=w.getFloatProperty("step",step);
+      page=w.getFloatProperty("page",page);
+      page_size=w.getFloatProperty("page_size",page_size);
+
    f.FunctionArg() << value;
    f.FunctionArg() << lower;
    f.FunctionArg() << upper;
    f.FunctionArg() << step;
    f.FunctionArg() << page;
    f.FunctionArg() << page_size;
+   }
 }
