@@ -1,4 +1,4 @@
-// $Id: arrow.cc,v 1.14 2001/10/11 14:07:24 christof Exp $
+// $Id: arrow.cc,v 1.17 2002/05/17 08:13:46 christof Exp $
 /*  glade--: C++ frontend for glade (Gtk+ User Interface Builder)
  *  Copyright (C) 1998  Christof Petig
  *  Copyright (C) 1999-2000  Adolf Petig GmbH & Co. KG, written by Christof Petig
@@ -38,7 +38,7 @@ const std::string Gtk_Arrow::TypeName(const Widget &w) const
 }
 
 const std::string Gtk_Arrow::IncludeName(const Widget &w) const
-{  return "gtk--/arrow.h";
+{  return Configuration.GtkmmIncludePath()+"arrow.h";
 }
 
 Gtk_Arrow::Gtk_Arrow()
@@ -46,6 +46,6 @@ Gtk_Arrow::Gtk_Arrow()
 }
 
 void Gtk_Arrow::ConstructionArgs(const Widget &w, CxxFile &f) const
-{  f.FunctionArg() << w.getString("arrow_type","GTK_ARROW_RIGHT");
-   f.FunctionArg() << w.getString("shadow_type","GTK_SHADOW_OUT");
+{  f.FunctionArg() << Gtkmm2Namespace(w.getProperty("arrow_type","GTK_ARROW_RIGHT"));
+   f.FunctionArg() << Gtkmm2Namespace(w.getProperty("shadow_type","GTK_SHADOW_OUT"));
 }
