@@ -1,4 +1,4 @@
-// $Id: checkbutton.cc,v 1.13 2001/10/11 14:07:24 christof Exp $
+// $Id: checkbutton.cc,v 1.20 2002/12/09 09:42:29 christof Exp $
 /*  glade--: C++ frontend for glade (Gtk+ User Interface Builder)
  *  Copyright (C) 1998  Christof Petig
  *  Copyright (C) 1999-2000  Adolf Petig GmbH & Co. KG, written by Christof Petig
@@ -19,6 +19,7 @@
  */
 
 #include "checkbutton.hh"
+typedef float gfloat;
 
 static Gtk_CheckButton Gtk_CheckButton(false);
 
@@ -26,8 +27,24 @@ const std::string Gtk_CheckButton::TypeName(const Widget &w) const
 {  return GtkPrefix()+"CheckButton";
 }
 const std::string Gtk_CheckButton::IncludeName(const Widget &w) const
-{  return "gtk--/checkbutton.h";
+{  return Configuration.GtkmmIncludePath()+"checkbutton.h";
 }
 Gtk_CheckButton::Gtk_CheckButton(bool base_class_init)
 {  if (!base_class_init) Writer["GtkCheckButton"]=this;
 }
+void Gtk_CheckButton::ConstructionArgs(const Widget &w, CxxFile &f) const
+{  if (w.hasProperty("label"))
+   {  std::string label=w.getProperty("label");
+      f.FunctionArg() << Configuration.Translatable(label);
+      // this is unique to check/radiobuttons
+      if (GTKMM1)
+      {  gfloat xalign=0.0;
+         gfloat yalign=0.5;
+         f.FunctionArg() << xalign;
+         f.FunctionArg() << yalign;
+      }
+      else
+      {  if (label.find('_')!=std::string::npos) f.FunctionArg() << "true";
+      }
+   }
+}
