@@ -1,4 +1,4 @@
-// $Id: frame.cc,v 1.20 2001/10/11 14:07:24 christof Exp $
+// $Id: frame.cc,v 1.25 2002/10/24 10:40:59 christof Exp $
 /*  glade--: C++ frontend for glade (Gtk+ User Interface Builder)
  *  Copyright (C) 1998  Christof Petig
  *  Copyright (C) 1999-2000  Adolf Petig GmbH & Co. KG, written by Christof Petig
@@ -29,6 +29,7 @@ class Gtk_Frame : public Gtk_Bin
 	virtual bool NeedExplicitCtor(const Widget &w) const;
 	virtual void ConstructionArgs(const Widget &w, CxxFile &f) const;
 	virtual void Configure(const Widget &w, CxxFile &f,const std::string &instance) const;
+	virtual void AddChildren(const Widget &w,CxxFile &f,const std::string &instance,const WriterBase &writer_for_subw, const Widget &widget_for_subw) const;
 };
 
 static Gtk_Frame Gtk_Frame;
@@ -38,7 +39,7 @@ const std::string Gtk_Frame::TypeName(const Widget &w) const
 }
 
 const std::string Gtk_Frame::IncludeName(const Widget &w) const
-{  return "gtk--/frame.h";
+{  return Configuration.GtkmmIncludePath()+"frame.h";
 }
 
 Gtk_Frame::Gtk_Frame()
@@ -47,19 +48,40 @@ Gtk_Frame::Gtk_Frame()
 
 void Gtk_Frame::ConstructionArgs(const Widget &w, CxxFile &f) const
 {  f.FunctionArg();
-   const std::string label(w.getString("label"));
-   if (label.size()) f << Configuration.Translatable(label);
+   if (w.hasProperty("label"))
+   {  const std::string label(w.getProperty("label"));
+      if (!label.empty()) f << Configuration.Translatable(label);
+   }
 }
 
 bool Gtk_Frame::NeedExplicitCtor(const Widget &w) const
-{  return w.hasTag("label") && w.getString("label").size(); }
+{  return w.hasProperty("label") && !w.getProperty("label").empty(); }
 
 void Gtk_Frame::Configure(const Widget &w, CxxFile &f,const std::string &instance) const
 {  Parent::Configure(w,f,instance);
-   float label_xalign(w.getFloat("label_xalign",0));
-   float label_yalign(w.getFloat("label_yalign",0));
-   std::string shadow_type(w.getString("shadow_type","GTK_SHADOW_ETCHED_IN"));
-   f.Statement() << instance << "set_label_align(" << label_xalign 
-   	<< ", " << label_yalign << ')';
-   f.Statement() << instance << "set_shadow_type(" << shadow_type << ')';
+   WriteEnumPropertyNS(w,f,instance, "shadow_type");
+   WriteFloatFloatProperty(w,f,instance, "label_xalign", "label_yalign", "label_align", false, 0);
 }
+
+void Gtk_Frame::AddChildren(const Widget &w,CxxFile &f,const std::string &instance,const WriterBase &writer_for_subw, const Widget &widget_for_subw) const
+{  // at least two widgets are needed for the new style
+   if (!Configuration.glade2 || w.begin()==w.end() || ++w.begin()==w.end()) 
+   {  Parent::AddChildren(w,f,instance,writer_for_subw,widget_for_subw);
+      return;
+   }
+
+   Widget::const_iterator label_item=w.begin();
+   Widget::const_iterator i=w.begin();
+   
+   const ChildParamList ch((*i).get_Child_params());
+   if (ch.getProperty("type")=="label_item") ++i;
+   else ++label_item;
+   
+   if ((*i).Class()!="Placeholder") AddChild(*i,f,instance);
+   
+   if ((*label_item).Class()!="Placeholder") 
+   {  f.Statement() << instance << "set_label_widget(" 
+   		<< Reference(*label_item) << ")";
+   }
+}
+
