@@ -1,4 +1,4 @@
-// $Id: optionmenu.cc,v 1.34 2001/10/11 14:07:24 christof Exp $
+// $Id: optionmenu.cc,v 1.41 2003/01/06 07:27:14 christof Exp $
 /*  glade--: C++ frontend for glade (Gtk+ User Interface Builder)
  *  Copyright (C) 1998  Christof Petig
  *  Copyright (C) 1999-2000  Adolf Petig GmbH & Co. KG, written by Christof Petig
@@ -39,6 +39,7 @@ public:
 	{  return false; }
 	virtual void ConstructionArgs(Widget const &w, CxxFile &f) const
 	{  f.FunctionArg(); }
+	virtual void AddChild(const Widget &w,CxxFile &f,const std::string &instance) const;
 };
 
 static Gtk_OptionMenu Gtk_OptionMenu;
@@ -48,7 +49,7 @@ const std::string Gtk_OptionMenu::TypeName(const Widget &w) const
 }
 
 const std::string Gtk_OptionMenu::IncludeName(const Widget &w) const
-{  return "gtk--/optionmenu.h";
+{  return Configuration.GtkmmIncludePath()+"optionmenu.h";
 }
 
 Gtk_OptionMenu::Gtk_OptionMenu()
@@ -56,20 +57,20 @@ Gtk_OptionMenu::Gtk_OptionMenu()
 }
 
 void Gtk_OptionMenu::GCInclude(const Widget &w, CxxFile &f) const
-{  if (w.hasTag("items")) 
-   {  f.Include("gtk--/menu.h");
-      f.Include("gtk--/menuitem.h");
+{  if (w.hasProperty("items")) 
+   {  f.Include(Configuration.GtkmmIncludePath()+"menu.h");
+      f.Include(Configuration.GtkmmIncludePath()+"menuitem.h");
    }
    Parent::GCInclude(w,f);
 }
 
-void Gtk_OptionMenu::AdditionalMemberVars(const Widget &w,CxxFile &f,bool container=false) const
+void Gtk_OptionMenu::AdditionalMemberVars(const Widget &w,CxxFile &f,bool container) const
 {  Parent::AdditionalMemberVars(w,f,container);
-   if (w.hasTag("items") && Configuration.optionmenu_create_enum)
+   if (w.hasProperty("items") && Configuration.optionmenu_create_enum)
    {  const std::string name(Configuration.InstanceName(w.Name()));
-      const std::string visibility(w.getString("cxx_visibility","private"));
-      const std::string items(w.getString("items","<none>\n"));      
-      if (visibility!="private" && items.size())
+      const std::string visibility(w.getProperty("cxx_visibility","private"));
+      const std::string items(w.getProperty("items","<none>\n"));      
+      if (visibility!="private" && !items.empty())
       {  if (visibility=="public") f.Public();
          else f.Protected();
          
@@ -78,7 +79,11 @@ void Gtk_OptionMenu::AdditionalMemberVars(const Widget &w,CxxFile &f,bool contai
          f.StartBlock();
          f.Definition() << "enum enum_t ";
          f.StartBlock();
+#ifdef __MINGW32__
+	 char buffer[items.size()+1];
+#else
          char *buffer=(char*)alloca(items.size()+1);
+#endif         
          memcpy(buffer,items.c_str(),items.size()+1);
          char *token,*stringp=buffer;
          while ((token=strsep(&stringp,"\n")))
@@ -95,10 +100,14 @@ void Gtk_OptionMenu::AdditionalMemberVars(const Widget &w,CxxFile &f,bool contai
 void Gtk_OptionMenu::Configure(const Widget &w, CxxFile &f,const std::string &instance) const
 {  Parent::Configure(w,f,instance);
    // XXX: Configure menu???
-   if (w.hasTag("items")) // append them
-   {  const std::string items(w.getString("items","<none>\n"));
+   if (w.hasProperty("items")) // append them
+   {  const std::string items(w.getProperty("items","<none>\n"));
       const std::string name(Configuration.InstanceName(w.Name()));
+#ifdef __MINGW32__
+      char buffer[items.size()+1];
+#else
       char *buffer=(char*)alloca(items.size()+1);
+#endif      
       memcpy(buffer,items.c_str(),items.size()+1);
       char *token,*stringp=buffer;
       f.StartBlock();
@@ -113,17 +122,29 @@ void Gtk_OptionMenu::Configure(const Widget &w, CxxFile &f,const std::string &in
          	<< Configuration.Translatable(token) << "))";
          f.Statement() << "_m->append(*_mi)";
          f.Statement() << "_mi->show()";
+         if (GTKMM1) f.Statement() << "_mi->set_user_data((gpointer)";
+         else f.Statement() << "_mi->property_user_data().set_value((gpointer)";
          if (Configuration.optionmenu_strings)
-            f.Statement() << "_mi->set_user_data((gpointer)" 
-            	<< Configuration.CString_WithQuotes(token) << ')';
+            f << Configuration.CString_WithQuotes(token) << ')';
          else
-            f.Statement() << "_mi->set_user_data((gpointer)" << index << ')';
+            f << index << ')';
          index++;
       }
       f.Statement() << name << "->set_menu(*_m)";
-      int initial_choice=w.getInt("initial_choice",0);
+      int initial_choice=w.getIntProperty("initial_choice",0);
       f.Statement() << name << "->set_history("<<initial_choice <<')';
       f.EndBlock();
    }
+   WriteIntProperty(w,f,instance, "history");
 }
 
+void Gtk_OptionMenu::AddChild(const Widget &w,CxxFile &f,const std::string &instance) const
+{  const std::string cn=w.ChildName();
+   if (!cn.empty())
+   {  std::cerr << "Warning: Gtk_Container::AddChild(" << w.Name() << ",,"
+           << instance << ") with child name " << cn
+           << ",\n\t" << TypeName(w) << "::IsSubwidget() might need fixing\n";
+   }
+   f.Statement() << instance << "set_menu(" << Reference(w) << ')';
+}
+  
\ No newline at end of file
