@@ -1,4 +1,4 @@
-// $Id: progressbar.cc,v 1.16 2001/10/11 14:07:24 christof Exp $
+// $Id: progressbar.cc,v 1.17 2002/05/14 08:25:45 christof Exp $
 /*  glade--: C++ frontend for glade (Gtk+ User Interface Builder)
  *  Copyright (C) 1998  Christof Petig
  *  Copyright (C) 1999-2000  Adolf Petig GmbH & Co. KG, written by Christof Petig
@@ -39,7 +39,7 @@ const std::string Gtk_ProgressBar::TypeName(const Widget &w) const
 }
 
 const std::string Gtk_ProgressBar::IncludeName(const Widget &w) const
-{  return "gtk--/progressbar.h";
+{  return Configuration.GtkmmIncludePath()+"progressbar.h";
 }
 
 Gtk_ProgressBar::Gtk_ProgressBar()
