@@ -1,4 +1,4 @@
-// $Id: radiobutton.cc,v 1.23 2001/12/27 12:29:14 christof Exp $
+// $Id: radiobutton.cc,v 1.34 2003/03/26 09:49:53 christof Exp $
 /*  glade--: C++ frontend for glade (Gtk+ User Interface Builder)
  *  Copyright (C) 1998  Christof Petig
  *  Copyright (C) 1999-2000 Adolf Petig GmbH & Co. KG, written by Christof Petig
@@ -18,23 +18,9 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#include "checkbutton.hh"
+#include "radiobutton.hh"
 #include <Cxx_Fileset.hh>
 
-class Gtk_RadioButton : public Gtk_CheckButton
-{public:
-	typedef Gtk_CheckButton Parent;
-	virtual const std::string TypeName(const Widget &w) const;
-	virtual const std::string IncludeName(const Widget &w) const;
-	Gtk_RadioButton();
-	virtual bool CantMemberConstruct(const Widget &w) const
-	{ return true; }
-	virtual void ConstructionArgs(const Widget &w, CxxFile &f) const;
-	virtual bool NeedExplicitCtor(const Widget &w) const
-	{ return true; }
-	virtual void CreatePointer(const Widget &w,CxxFile &f) const;
-};
-
 static Gtk_RadioButton Gtk_RadioButton;
 
 const std::string Gtk_RadioButton::TypeName(const Widget &w) const
@@ -42,23 +28,41 @@ const std::string Gtk_RadioButton::TypeName(const Widget &w) const
 }
 
 const std::string Gtk_RadioButton::IncludeName(const Widget &w) const
-{  return "gtk--/radiobutton.h";
+{  return Configuration.GtkmmIncludePath()+"radiobutton.h";
 }
 
 Gtk_RadioButton::Gtk_RadioButton()
 {  Writer["GtkRadioButton"]=this;
+   Writer["radiobutton"]=this; // is this ever needed? 2.0 ???
+   Writer["radio"]=this;  // 1.1.3
 }
 
 void Gtk_RadioButton::ConstructionArgs(const Widget &w, CxxFile &f) const
 {  f.FunctionArg();
-   f << "_RadioBGroup_" << Configuration.InstanceName(w.getString("group"));
-   if (w.hasTag("label"))
-      f.FunctionArg() << Configuration.Translatable(w.getString("label"));
+   f << "_RadioBGroup_" << Configuration.InstanceName(w.getProperty("group",w.Name()));
+   // checkbutton's arguments
+   Parent::ConstructionArgs(w,f);
 }
 
 void Gtk_RadioButton::CreatePointer(const Widget &w,CxxFile &f) const
-{  if (Cxx_Fileset::NeedToDeclareRadioGroup("B:"+w.getString("group","")))
-      f.Declaration() << "Gtk::RadioButton::Group _RadioBGroup_" 
-		<< Configuration.InstanceName(w.getString("group",""));
+{  ToolBar_vars(w,f); // create local Group var
    Parent::CreatePointer(w,f);
 }
+
+void Gtk_RadioButton::ToolBarElemArgs(const Widget &w, CxxFile &f) const
+{  f.FunctionArg() << "_RadioBGroup_" << w.getProperty("group",w.Name());
+   Parent::ToolBarElemArgs(w,f);
+}
+
+void Gtk_RadioButton::ToolBar_vars(const Widget &w,CxxFile &f) const
+{  std::string group=w.getProperty("group",w.Name());
+   if (Cxx_Fileset::NeedToDeclareRadioGroup("B:"+group))
+      f.Declaration() << "Gtk::RadioButton::Group _RadioBGroup_" 
+		<< Configuration.InstanceName(group);
+}
+
+void Gtk_RadioButton::GCInclude(const Widget &w, CxxFile &f) const
+{  f.Include(Configuration.GtkmmIncludePath()+"radiobutton.h");
+   Parent::GCInclude(w,f);
+}
+
