@@ -1,4 +1,4 @@
-// $Id: range.cc,v 1.33 2001/10/11 14:07:24 christof Exp $
+// $Id: range.cc,v 1.38 2002/06/17 08:12:26 christof Exp $
 /*  glade--: C++ frontend for glade (Gtk+ User Interface Builder)
  *  Copyright (C) 1998  Christof Petig
  *  Copyright (C) 1999-2000 Adolf Petig GmbH & Co. KG, written by Christof Petig
@@ -22,24 +22,23 @@
 
 static Gtk_Range Gtk_Range(false);
 
-#define COPY_TAG(name) adjustment.push_back(Tag(name,w.getString(name)))
+#define COPY_TAG(name) w2.setProperty((name),w.getProperty(name))
 const Widget Gtk_Range::get_adjustment(const Widget &w) const
 {  static Tag adjustment("widget");
    std::string name(Configuration.InstanceName(w.Name())+"_adj");
-   if (adjustment.getString("name")==name)
+   if (Widget(adjustment).Name()==name)
    {  return Widget(adjustment);
    }
-   adjustment=Tag("widget");
-   adjustment.push_back(Tag("class","GtkAdjustment"));
-   adjustment.push_back(Tag("name",name));
+   adjustment=createWidgetTag("GtkAdjustment", name);
+   Widget w2(adjustment);
    COPY_TAG("value");
    COPY_TAG("lower");
    COPY_TAG("upper");
    COPY_TAG("step");
    COPY_TAG("page");
    COPY_TAG("page_size");
-   adjustment.push_back(Tag(CXX_IS_MANAGED,"true"));
-   return Widget(adjustment);
+   w2.setProperty(CXX_IS_MANAGED);
+   return w2;
 }
 
 const std::string Gtk_Range::TypeName(const Widget &w) const
@@ -53,7 +52,7 @@ void Gtk_Range::GCInclude(const Widget &w, CxxFile &f) const
 }
 
 const std::string Gtk_Range::IncludeName(const Widget &w) const
-{  return "gtk--/range.h";
+{  return Configuration.GtkmmIncludePath()+"range.h";
 }
 
 Gtk_Range::Gtk_Range(bool base_class_init)
@@ -62,9 +61,7 @@ Gtk_Range::Gtk_Range(bool base_class_init)
 
 void Gtk_Range::Configure(const Widget &w, CxxFile &f,const std::string &instance) const
 {  Parent::Configure(w,f,instance);
-   std::string update_policy=w.getString("update_policy","GTK_UPDATE_ALWAYS");
-   if (update_policy!="GTK_UPDATE_ALWAYS")
-      f.Statement() << instance << "set_update_policy(" << update_policy << ')';
+   WriteEnumPropertyNS(w,f,instance, "update_policy");
 }
 
 bool Gtk_Range::CantMemberConstruct(const Widget &w) const
