@@ -1,4 +1,4 @@
-// $Id: scale.cc,v 1.22 2001/10/11 14:07:24 christof Exp $
+// $Id: scale.cc,v 1.27 2002/07/18 10:09:46 christof Exp $
 /*  glade--: C++ frontend for glade (Gtk+ User Interface Builder)
  *  Copyright (C) 1998  Christof Petig
  *  Copyright (C) 1999-2000 Adolf Petig GmbH & Co. KG, written by Christof Petig
@@ -37,7 +37,7 @@ const std::string Gtk_Scale::TypeName(const Widget &w) const
 }
 
 const std::string Gtk_Scale::IncludeName(const Widget &w) const
-{  return "gtk--/scale.h";
+{  return Configuration.GtkmmIncludePath()+"scale.h";
 }
 
 Gtk_Scale::Gtk_Scale()
@@ -47,10 +47,7 @@ Gtk_Scale::Gtk_Scale()
 
 void Gtk_Scale::Configure(const Widget &w, CxxFile &f,const std::string &instance) const
 {  Parent::Configure(w,f,instance);
-   int digits=w.getInt("digits",0);
-   f.Statement() << instance << "set_digits(" << digits << ')';
-   bool draw_value=w.getInt("draw_value",true);
-   f.Statement() << instance << "set_draw_value(" << PRINT_BOOL(draw_value) << ')';
-   std::string value_pos=w.getString("value_pos","GTK_POS_TOP");
-   f.Statement() << instance << "set_value_pos(" << value_pos << ')';
+   WriteIntProperty(w,f,instance, "digits");
+   WriteBoolProperty(w,f,instance, "draw_value");
+   WriteEnumPropertyNS(w,f,instance, "value_pos");
 }
