@@ -1,4 +1,4 @@
-// $Id: scrolledwindow.cc,v 1.21 2001/10/11 14:07:24 christof Exp $
+// $Id: scrolledwindow.cc,v 1.26 2002/12/13 19:14:44 christof Exp $
 /*  glade--: C++ frontend for glade (Gtk+ User Interface Builder)
  *  Copyright (C) 1998  Christof Petig
  *  Copyright (C) 1999-2000 Adolf Petig GmbH & Co. KG, written by Christof Petig
@@ -40,7 +40,7 @@ const std::string Gtk_ScrolledWindow::TypeName(const Widget &w) const
 }
 
 const std::string Gtk_ScrolledWindow::IncludeName(const Widget &w) const
-{  return "gtk--/scrolledwindow.h";
+{  return Configuration.GtkmmIncludePath()+"scrolledwindow.h";
 }
 
 Gtk_ScrolledWindow::Gtk_ScrolledWindow()
@@ -49,15 +49,16 @@ Gtk_ScrolledWindow::Gtk_ScrolledWindow()
 
 void Gtk_ScrolledWindow::Configure(const Widget &w, CxxFile &f,const std::string &instance) const
 {  Parent::Configure(w,f,instance);
-//   std::string shadow_type(w.getString("shadow_type","GTK_SHADOW_NONE"));
-//   and what to do with this?
-//   f.Statement() << instance << "set_shadow_type(" << shadow_type << ')';
-   std::string hscrollbar_policy(w.getString("hscrollbar_policy","GTK_POLICY_AUTOMATIC"));
-   std::string vscrollbar_policy(w.getString("vscrollbar_policy","GTK_POLICY_AUTOMATIC"));
-   f.Statement() << instance << "set_policy(" << hscrollbar_policy
-   	<< ", " << vscrollbar_policy << ')';
-   // I don't know how to set update policies ... sorry
-   // glade does this via 
-   // gtk_range_set_update_policy (GTK_RANGE (GTK_SCROLLED_WINDOW (scrolledwindow1)
-   // ->hscrollbar), GTK_POLICY_NEVER);
+   if (GTKMM2)
+   {  std::string shadow_type(w.getProperty("shadow_type","GTK_SHADOW_NONE"));
+      f.Statement() << instance << "set_shadow_type(" << Gtkmm2Namespace(shadow_type) << ')';
+   }
+   if (w.hasProperty("hscrollbar_policy") || w.hasProperty("vscrollbar_policy"))
+   {  std::string hscrollbar_policy(w.getProperty("hscrollbar_policy","GTK_POLICY_AUTOMATIC"));
+      std::string vscrollbar_policy(w.getProperty("vscrollbar_policy","GTK_POLICY_AUTOMATIC"));
+      f.Statement() << instance << "set_policy(" << Gtkmm2Namespace(hscrollbar_policy)
+   	<< ", " << Gtkmm2Namespace(vscrollbar_policy) << ')';
+   }
+   // we might use set_placement ...
+   WriteEnumPropertyNS(w,f,instance, "window_placement", true);
 }
