@@ -1,4 +1,4 @@
-// $Id: spinbutton.cc,v 1.31 2001/10/11 14:07:24 christof Exp $
+// $Id: spinbutton.cc,v 1.38 2002/12/19 08:55:54 christof Exp $
 /*  glade--: C++ frontend for glade (Gtk+ User Interface Builder)
  *  Copyright (C) 1998  Christof Petig
  *  Copyright (C) 1999-2000  Adolf Petig GmbH & Co. KG, written by Christof Petig
@@ -39,24 +39,26 @@ public:
 
 static Gtk_SpinButton Gtk_SpinButton;
 
-#define COPY_TAG(name) adjustment.push_back(Tag(name,w.getString(name)))
+#define COPY_TAG(name) w2.setProperty((name),w.getProperty(name))
 const Widget Gtk_SpinButton::get_adjustment(const Widget &w) const
 {  static Tag adjustment("widget");
    std::string name(Configuration.InstanceName(w.Name())+"_adj");
-   if (adjustment.getString("name")==name)
+   if (Widget(adjustment).Name()==name)
    {  return Widget(adjustment);
    }
-   adjustment=Tag("widget");
-   adjustment.push_back(Tag("class","GtkAdjustment"));
-   adjustment.push_back(Tag("name",name));
-   COPY_TAG("value");
-   COPY_TAG("lower");
-   COPY_TAG("upper");
-   COPY_TAG("step");
-   COPY_TAG("page");
-   COPY_TAG("page_size");
-   adjustment.push_back(Tag(CXX_IS_MANAGED,"true"));
-   return Widget(adjustment);
+   adjustment=createWidgetTag("GtkAdjustment", name);
+   Widget w2(adjustment);
+   if (Configuration.glade2) COPY_TAG("adjustment");
+   else
+   {  COPY_TAG("value");
+      COPY_TAG("lower");
+      COPY_TAG("upper");
+      COPY_TAG("step");
+      COPY_TAG("page");
+      COPY_TAG("page_size");
+   }
+   w2.setProperty(CXX_IS_MANAGED);
+   return w2;
 }
 
 const std::string Gtk_SpinButton::TypeName(const Widget &w) const
@@ -64,7 +66,7 @@ const std::string Gtk_SpinButton::TypeName(const Widget &w) const
 }
 
 const std::string Gtk_SpinButton::IncludeName(const Widget &w) const
-{  return "gtk--/spinbutton.h";
+{  return Configuration.GtkmmIncludePath()+"spinbutton.h";
 }
 
 // we rely on adjustment getting a local variable
@@ -96,26 +98,16 @@ void Gtk_SpinButton::DestroyPointer(const Widget &w,CxxFile &f) const
 void Gtk_SpinButton::ConstructionArgs(const Widget &w, CxxFile &f) const
 {  Widget w2(get_adjustment(w));
    f.FunctionArg() << Reference(w2);
-   f.FunctionArg() << w.getInt("climb_rate",0);
-   f.FunctionArg() << w.getInt("digits",0);
+   f.FunctionArg() << w.getIntProperty("climb_rate",0);
+   f.FunctionArg() << w.getIntProperty("digits",0);
 }
 
 void Gtk_SpinButton::Configure(const Widget &w, CxxFile &f,const std::string &instance) const
 {  Parent::Configure(w,f,instance);
-   std::string update_policy=w.getString("update_policy","GTK_UPDATE_ALWAYS");
-   if (update_policy!="GTK_UPDATE_ALWAYS")
-      f.Statement() << instance << "set_update_policy(" << update_policy << ')';
-   bool numeric(w.getBool("numeric"));
-   if (numeric)
-      f.Statement() << instance << "set_numeric(" << PRINT_BOOL(numeric) << ')';
-   int digits(w.getInt("digits"));
-   if (digits)
-      f.Statement() << instance << "set_digits(" << digits << ')';
-   bool wrap(w.getBool("wrap"));
-   if (wrap)
-      f.Statement() << instance << "set_wrap(" << PRINT_BOOL(wrap) << ')';
-   bool snap(w.getBool("snap"));
-   if (snap)
-      f.Statement() << instance << "set_snap_to_ticks(" << PRINT_BOOL(snap) << ')';
-   // shadow type
+   WriteEnumPropertyNS(w,f,instance, "update_policy");
+   WriteBoolProperty(w,f,instance, "numeric");
+   WriteIntProperty(w,f,instance, "digits");
+   WriteBoolProperty(w,f,instance, "wrap");
+   WriteBoolProperty(w,f,instance, "snap", false, "snap_to_ticks");
+   WriteEnumPropertyNS(w,f,instance, "shadow_type");
 }
