@@ -1,4 +1,4 @@
-// $Id: statusbar.cc,v 1.17 2001/10/11 14:07:25 christof Exp $
+// $Id: statusbar.cc,v 1.18 2002/05/14 08:25:45 christof Exp $
 /*  glade--: C++ frontend for glade (Gtk+ User Interface Builder)
  *  Copyright (C) 1998  Christof Petig
  *  Copyright (C) 1999-2000 Adolf Petig GmbH & Co. KG, written by Christof Petig
@@ -39,7 +39,7 @@ const std::string Gtk_StatusBar::TypeName(const Widget &w) const
 }
 
 const std::string Gtk_StatusBar::IncludeName(const Widget &w) const
-{  return "gtk--/statusbar.h";
+{  return Configuration.GtkmmIncludePath()+"statusbar.h";
 }
 
 Gtk_StatusBar::Gtk_StatusBar()
