@@ -1,4 +1,4 @@
-// $Id: text.cc,v 1.23 2001/10/11 14:07:25 christof Exp $
+// $Id: text.cc,v 1.28 2002/06/15 13:22:14 christof Exp $
 /*  glade--: C++ frontend for glade (Gtk+ User Interface Builder)
  *  Copyright (C) 1998  Christof Petig
  *  Copyright (C) 1999-2000 Adolf Petig GmbH & Co. KG, written by Christof Petig
@@ -31,7 +31,8 @@ class Gtk_Text : public Gtk_Editable
 	{  return false; }
 	virtual void ConstructionArgs(Widget const &w, CxxFile &f) const
 	{  f.FunctionArg(); }
-	virtual const std::string SignalHandlerArgs(const Widget &w,const std::string &signal,std::string &rettype) const;
+	virtual const std::string SignalHandlerArgs(const Widget &w,const std::string &signal,std::string &rettype,std::string &scope) const;
+	virtual void ApplyPreferences(Tag &t) const;
 };
 
 static Gtk_Text Gtk_Text;
@@ -41,7 +42,7 @@ const std::string Gtk_Text::TypeName(const Widget &w) const
 }
 
 const std::string Gtk_Text::IncludeName(const Widget &w) const
-{  return "gtk--/text.h";
+{  return Configuration.GtkmmIncludePath()+"text.h";
 }
 
 Gtk_Text::Gtk_Text()
@@ -50,14 +51,11 @@ Gtk_Text::Gtk_Text()
 
 void Gtk_Text::Configure(const Widget &w, CxxFile &f,const std::string &instance) const
 {  Parent::Configure(w,f,instance);
-   bool editable=w.getBool("editable",false);
-   f.Statement() << instance << "set_editable(" << PRINT_BOOL(editable) << ')';
-   bool word_wrap=w.getBool("word_wrap",false);
-   f.Statement() << instance << "set_word_wrap(" << PRINT_BOOL(word_wrap) << ')';
-   int point(w.getInt("point",0));
-   f.Statement() << instance << "set_point(" << point << ')';
-   std::string text(w.getString("text"));
-   if (text.size())
+   WriteBoolProperty(w,f,instance, "editable");
+   WriteBoolProperty(w,f,instance, "word_wrap");
+   WriteIntProperty(w,f,instance, "point");
+   std::string text(w.getProperty("text"));
+   if (!text.empty())
    {  f.StartBlock().Declaration("gint _dummy(0)");
       f.Statement() << instance << "insert_text(" 
       	<< Configuration.Translatable(text) 
@@ -69,8 +67,20 @@ void Gtk_Text::Configure(const Widget &w, CxxFile &f,const std::string &instance
    }
 }
 
-const std::string Gtk_Text::SignalHandlerArgs(const Widget &w,const std::string &signal,std::string &rettype) const
-{  if (signal=="set_scroll_adjustments")
+const std::string Gtk_Text::SignalHandlerArgs(const Widget &w,const std::string &signal,std::string &rettype,std::string &scope) const
+{  scope=Gtk_Text::TypeName(w);
+   if (signal=="set_scroll_adjustments")
       return "GtkAdjustment *p0, GtkAdjustment *p1";
-   return Parent::SignalHandlerArgs(w,signal,rettype);
+   return Parent::SignalHandlerArgs(w,signal,rettype,scope);
 }
+
+void Gtk_Text::ApplyPreferences(Tag &t) const
+{  if (GTKMM2)
+   {  static_cast<GladeTag*>(&t)->setWidgetType("GtkTextView");
+      Widget w(&t);
+      LookupWriter(w).ApplyPreferences(t);
+   }
+   else
+      Parent::ApplyPreferences(t);
+}
+
