@@ -1,4 +1,4 @@
-// $Id: togglebutton.cc,v 1.24 2001/10/11 14:07:25 christof Exp $
+// $Id: togglebutton.cc,v 1.31 2003/04/24 08:20:41 christof Exp $
 /*  glade--: C++ frontend for glade (Gtk+ User Interface Builder)
  *  Copyright (C) 1998  Christof Petig
  *  Copyright (C) 1999-2000 Adolf Petig GmbH & Co. KG, written by Christof Petig
@@ -27,28 +27,27 @@ const std::string Gtk_ToggleButton::TypeName(const Widget &w) const
 }
 
 const std::string Gtk_ToggleButton::IncludeName(const Widget &w) const
-{  return "gtk--/togglebutton.h";
+{  return Configuration.GtkmmIncludePath()+"togglebutton.h";
 }
 
 Gtk_ToggleButton::Gtk_ToggleButton(bool base_class_init)
-{  if (!base_class_init) Writer["GtkToggleButton"]=this;
+{  if (!base_class_init)
+   {  Writer["GtkToggleButton"]=this;
+      Writer["togglebutton"]=this;
+      Writer["toggle"]=this;
+   }
 }
 
 void Gtk_ToggleButton::Configure(const Widget &w, CxxFile &f,const std::string &instance) const
 {  Parent::Configure(w,f,instance);
-   bool draw_indicator=w.getBool("draw_indicator",true);
-   if (!draw_indicator)
-      f.Statement() << instance << "set_mode(" << PRINT_BOOL(draw_indicator) << ')';
-   bool active=w.getBool("active",true);
-   if (active)
-      f.Statement() << instance << "set_active(" << PRINT_BOOL(active) << ')';
+   WriteBoolProperty(w,f,instance, "draw_indicator", false, "mode");
+   WriteBoolProperty(w,f,instance, "active");
 }
 
 bool Gtk_ToggleButton::isInternalMethod(const Widget &w,std::string &method,const std::string &args,std::string &scope,bool &is_signal) const
-{  const char *prefix("gtk_toggle_button_");
+{  const char prefix[]="gtk_toggle_button_";
    const int plen(sizeof(prefix)-1);
-   if (!strncmp(prefix,method.c_str(),plen)) 
-      method=std::string(method.c_str()+plen);
+   if (method.substr(0,plen)==prefix) method=method.substr(plen);
    if ((method=="set_mode" && matches("bool\\ \\_",args))
         || (method=="set_active" && matches("bool\\ \\_",args)))
    {  scope=Gtk_ToggleButton::TypeName(w);
