@@ -1,4 +1,4 @@
-// $Id: toolbar.cc,v 1.48 2001/10/22 06:20:38 christof Exp $
+// $Id: toolbar.cc,v 1.74 2003/11/03 07:39:00 christof Exp $
 /*  glade--: C++ frontend for glade (Gtk+ User Interface Builder)
  *  Copyright (C) 1998  Christof Petig
  *  Copyright (C) 1999-2000 Adolf Petig GmbH & Co. KG, written by Christof Petig
@@ -19,9 +19,11 @@
  */
 
 #include "container.hh"
+#include "button.hh"
+#include <typeinfo>
 
 class Gtk_Toolbar : public Gtk_Container
-{	bool isXPM(const std::string &file) const;
+{
 public:
 	typedef Gtk_Container Parent;
 	virtual const std::string TypeName(const Widget &w) const;
@@ -30,7 +32,7 @@ public:
 	virtual bool NeedExplicitCtor(const Widget &w) const
 	{  return true; }
 	virtual void ConstructionArgs(const Widget &w, CxxFile &f) const;
-	virtual void AddChildren(const Widget &w,CxxFile &f,const std::string &instance) const;
+	virtual void AddChildren(const Widget &w,CxxFile &f,const std::string &instance,const WriterBase &writer_for_subw, const Widget &widget_for_subw) const;
 	virtual void Configure(const Widget &w, CxxFile &f,const std::string &instance) const;
 //	virtual bool CantMemberConstruct(const Widget &w) const
 //	{  return true; }
@@ -42,6 +44,10 @@ public:
 	virtual void GCInclude(const Widget &w, CxxFile &f) const;
 	virtual const std::string Instance(const Widget &parent,const Widget &w2,Subwidget sw=SW_Unknown) const;
 	virtual bool isInternalMethod(const Widget &w,std::string &method,const std::string &args,std::string &scope,bool &is_signal) const;
+	
+	// new style API
+	void CreatePointer_depending(const Widget &w,CxxFile &f) const;
+	void PushToolBarElem(const Widget &w,CxxFile &f, const std::string &instance, const Widget &tb) const;
 };
 
 static Gtk_Toolbar Gtk_Toolbar;
@@ -51,7 +57,7 @@ const std::string Gtk_Toolbar::TypeName(const Widget &w) const
 }
 
 const std::string Gtk_Toolbar::IncludeName(const Widget &w) const
-{  return "gtk--/toolbar.h";
+{  return Configuration.GtkmmIncludePath()+"toolbar.h";
 }
 
 Gtk_Toolbar::Gtk_Toolbar()
@@ -59,129 +65,57 @@ Gtk_Toolbar::Gtk_Toolbar()
 }
 
 void Gtk_Toolbar::ConstructionArgs(const Widget &w, CxxFile &f) const
-{  std::string orientation=w.getString("orientation","GTK_ORIENTATION_HORIZONTAL");
-   std::string style=w.getString("type","GTK_TOOLBAR_BOTH");
-   
+{  
+  if (GTKMM1)
+  {
+   std::string orientation=w.getProperty("orientation","GTK_ORIENTATION_HORIZONTAL");
+   std::string style=w.getProperty("type","GTK_TOOLBAR_BOTH");
+
    f.FunctionArg() << orientation;
    f.FunctionArg() << style;
+  }
+  else f.FunctionArg();
 }
 
  // toolbar.tools.push_back(WidgetElem(widget[,tooltip_text,private_text]));
-void Gtk_Toolbar::AddChildren(const Widget &w,CxxFile &f,const std::string &instance) const
-{
-#if 0
-   for (Widget::const_iterator i=w.begin();i!=w.end();++i)
-   {  if ((*i).Class()!="Placeholder")
-      {const Widget w2(*i);
-       const std::string clss(w2.Class());
-       if (IsSubwidget(w,*i)==not_Subwidget || (clss!="GtkButton" 
-       			&& clss!="GtkToggleButton" && clss!="GtkRadioButton"))
-       {f.Statement() << instance << "tools().push_back(Gtk::Toolbar_Helpers::WidgetElem("
-       		<< Reference(w2) << "))";
-       	// tooltip_text?
-       }        
-      }
-   }
-#endif   
+void Gtk_Toolbar::AddChildren(const Widget &w,CxxFile &f,const std::string &instance,const WriterBase &writer_for_subw, const Widget &widget_for_subw) const
+{  // CreatePointer_depending does the work
 }
 
 void Gtk_Toolbar::CreatePointer_Toplevel(const Widget &w,CxxFile &f) const
 {  Parent::CreatePointer_Toplevel(w,f);
-   for (Widget::const_iterator i=w.begin();i!=w.end();++i)
-   {  if ((*i).Class()!="Placeholder")
-      {const Widget w2(*i);
-       if (IsSubwidget(w,*i)!=not_Subwidget && ((*i).ChildName()=="Toolbar:button"))
-       { const std::string label(w2.getString("label"));
-         const std::string stock_pixmap(w2.getString("stock_pixmap"));
-         const std::string icon(w2.getString("icon"));
-         
-	// we can't use C++ because we don't want to connect right now
-	// and what about the new calling style?
-	
-	// GTK_TOOLBAR_CHILD_WIDGET
-	
-	 const WriterBase &wr2(LookupWriter(w2));
-	 std::string gtk_macro(wr2.GtkCast(w2)),
-	 	toolbar_type("GTK_TOOLBAR_CHILD_BUTTON");
-	 	
-         if (w2.Class()=="GtkButton") {}
-         else if (w2.Class()=="GtkToggleButton") 
-            toolbar_type="GTK_TOOLBAR_CHILD_TOGGLEBUTTON";
-         else if (w2.Class()=="GtkRadioButton") 
-            toolbar_type="GTK_TOOLBAR_CHILD_RADIOBUTTON";
-         else
-            continue; // we will add it later
-            
-         const ChildParamList ch=w2.get_Child_params();
-         if (ch.getBool("new_group"))
-         {  f.Statement() << instance(w) << "tools().push_back(Gtk::Toolbar_Helpers::Space())";
-         }
-         if (w2.getString("cxx_visibility","private")=="private") // && CanBeManaged(w2))
-	    f.Declaration() << wr2.TypeName(w2) << " *";
-	 else 
-	    f.Statement();
-         f << Configuration.InstanceName(w2.Name()) << " = ";
-         f << "Gtk::wrap((" << wr2.GtkName(w2) << "*)";
-         w2.markManaged();
-         f << "gtk_toolbar_append_element" 
-         	   "(GTK_TOOLBAR("<< instance(w) << "gtkobj()), "
-         	<< toolbar_type << ", "
-         	   "0" // widget
-         	   ", " << Configuration.Translatable(label) << ", " // text
-         	   "0, " // tooltip text
-         	   "0, " // tooltip_private_text
-         	;
-         if (stock_pixmap.size())
-             f << "GTK_WIDGET(gnome_stock_pixmap_widget(GTK_WIDGET(" 
-         	<< instance(w) << "gtkobj()), " // window, is this wrong?
-                << stock_pixmap << ")), ";
-         else if (icon.size() && isXPM(icon))
-             f << "GTK_WIDGET(manage(new "<< GtkPrefix() << "Pixmap(" 
-             	<< Naming::ToCIdentifier(icon) << "))->gtkobj()), ";
-         else if (icon.size())
-         {  std::cerr << "non .xpm icons are not implemented, yet\n";
-            f << "0, ";
-         }
-         else f << "0, "; // pixmap
-         f << "0, 0))"; // callback, user_data
-       }
-       else // yes I know, this is AddChildren but then it would get out
-       	// of order
-       { f.Statement() << instance(w)
-       		<< "tools().push_back(Gtk::Toolbar_Helpers::WidgetElem("
-       		<< Reference(w2) << "))";
-       	// tooltip_text?
-       }        
-      }
-   }
 }
 
 Subwidget Gtk_Toolbar::IsSubwidget(const Widget &w,const Widget &ch) const
-{  if (ch.ChildName()=="Toolbar:button") 
+{  if (ch.ChildName()=="Toolbar:button" || ch.Class()=="button"
+	|| ch.Class()=="togglebutton" || ch.Class()=="radiobutton" 
+	|| ch.Class()=="toggle" || ch.Class()=="radio") 
    {  if (ch.begin()!=ch.end()) return not_Subwidget; // constructs itself
       return is_Subwidget_all;
    }
-//   std::cerr << "Toolbar with strange child "<< ch.Name() << '\n';
    return not_Subwidget;
 }
 
 void Gtk_Toolbar::Configure(const Widget &w, CxxFile &f,const std::string &instance) const
 {  Parent::Configure(w,f,instance);
-   int space_size(w.getInt("space_size"));
-   if (space_size)
-      f.Statement() << instance << "set_space_size(" << space_size << ')';
-   bool tooltips(w.getBool("tooltips"));
-   if (tooltips)
-      f.Statement() << instance << "set_tooltips(" << PRINT_BOOL(tooltips) << ')';
-   const std::string relief(w.getString("relief","GTK_RELIEF_NORMAL"));
+   if (GTKMM1) WriteIntProperty(w,f,instance, "space_size");
+   WriteBoolProperty(w,f,instance, "tooltips");
+
+   const std::string relief(w.getProperty("relief","GTK_RELIEF_NORMAL"));
    if (relief!="GTK_RELIEF_NORMAL")
-      f.Statement() << instance << "set_button_relief(" << relief << ')';
-   const std::string space_style(w.getString("space_style","GTK_TOOLBAR_SPACE_EMPTY"));
-   if (space_style!="GTK_TOOLBAR_SPACE_EMPTY")
-      f.Statement() << instance << "set_space_style(" << space_style << ')';
+      f.Statement() << instance << "set_button_relief(" << Gtkmm2Namespace(relief) << ')';
+
+  if (GTKMM1)
+  {  WriteEnumPropertyNS(w,f,instance, "space_style");
+  }
+  else
+  {  WriteEnumPropertyNS(w,f,instance, "toolbar_style");
+     WriteEnumPropertyNS(w,f,instance, "orientation");
+     WriteEnumPropertyNS(w,f,instance, "icon_size");
+  }
 }
 
-void Gtk_Toolbar::AdditionalMemberVars(const Widget &w,CxxFile &f,bool container=false) const
+void Gtk_Toolbar::AdditionalMemberVars(const Widget &w,CxxFile &f,bool container) const
 {  Parent::AdditionalMemberVars(w,f,container);
                      
    for (Widget::const_iterator i=w.begin();i!=w.end();++i)
@@ -207,9 +141,7 @@ void Gtk_Toolbar::ApplyPreferences(Tag &t) const
    Widget w(&t);
    for (Widget::iterator i=w.begin();i!=w.end();++i)
    {  if ((*i).Class()!="Placeholder" && IsSubwidget(w,*i)!=not_Subwidget)
-      {  Widget w2(*i);
-         WriterBase::ApplyPreferences(const_cast<Tag&>(w2.getTag()));
-         if (!Configuration.gnome_support && w2.hasTag("stock_pixmap"))
+      {  if (GTKMM1 && !Configuration.gnome_support && (*i).hasProperty("stock_pixmap"))
          {  Configuration.gnome_support=true;
             std::cerr << "Stock Pixmaps need gnome_support, please turn it on\n";
          }
@@ -218,19 +150,32 @@ void Gtk_Toolbar::ApplyPreferences(Tag &t) const
 }
 
 void Gtk_Toolbar::GCInclude(const Widget &w, CxxFile &f) const
-{  if (Configuration.gnome_support) // cheap assertion, doesn't hurt
-      f.Include("libgnomeui/gnome-stock.h");
-   // look for .xpm items in the buttons to include
+{  // look for .xpm items in the buttons to include
    for (Widget::const_iterator i=w.begin();i!=w.end();++i)
-   {  if ((*i).Class()!="Placeholder" && ((*i).ChildName()=="Toolbar:button"))
+   {  if ((*i).Class()!="Placeholder" && IsSubwidget(w,*i)!=not_Subwidget)
       {const Widget w2(*i);
-       if (IsSubwidget(w,*i)!=not_Subwidget)
-       {  const std::string icon(w2.getString("icon"));
-          if (icon.size() && isXPM(icon))
-          {  f.Include("gtk--/pixmap.h");
+       if (w2.hasProperty("stock_pixmap"))
+       {  if (GTKMM1 && Configuration.gnome_support)
+             f.Include("libgnomeui/gnome-stock.h");
+          else if (GTKMM2)
+          {  f.Include(Configuration.GtkmmIncludePath()+"image.h");
+             f.Include(Configuration.GtkmmIncludePath()+"stockid.h");
+          }
+       }
+       if (GTKMM1 && w2.hasProperty("icon"))
+       {  const std::string icon(w2.getProperty("icon"));
+          if (!icon.empty() && !xpmname(icon).empty())
+          {  f.Include(Configuration.GtkmmIncludePath()+"pixmap.h");
              f.Include(Configuration.CString(Configuration.pixmap_dir_relative_to_src+"/"+icon),true);
           }
        }
+       else if (GTKMM2 && w2.hasProperty("icon"))
+       {  const Widget image=static_cast<const Gtk_Button*>(Writer["GtkButton"])
+		->get_image(w2);
+          const WriterBase &wr=LookupWriter(image);
+	  wr.GHInclude(image,f);
+	  wr.GCInclude(image,f);
+       }
       }
    }
    Parent::GCInclude(w,f);
@@ -242,11 +187,10 @@ const std::string Gtk_Toolbar::Instance(const Widget &parent,const Widget &w2, S
 }
 
 bool Gtk_Toolbar::isInternalMethod(const Widget &w,std::string &method,const std::string &args,std::string &scope,bool &is_signal) const
-{  const char *prefix("gtk_toolbar_");
+{  const char prefix[]="gtk_toolbar_";
    const int plen(sizeof(prefix)-1);
-   if (!strncmp(prefix,method.c_str(),plen))
-      method=std::string(method.c_str()+plen);
-// not in 1.2   if (((method=="append_space" ||method=="prepend_space") && !args.size())
+   if (method.substr(0,plen)==prefix) method=method.substr(plen);
+// not in 1.2   if (((method=="append_space" ||method=="prepend_space") && args.empty())
    if (((method=="set_tooltips") && matches("bool\\ \\_",args))
 	|| ((method=="set_space_size") 
 		&& (matches("gint\\ \\_",args)||matches("int\\ \\_",args))))
@@ -257,8 +201,55 @@ bool Gtk_Toolbar::isInternalMethod(const Widget &w,std::string &method,const std
    return Parent::isInternalMethod(w,method,args,scope,is_signal);
 }
 
-bool Gtk_Toolbar::isXPM(const std::string &file) const
-{  size_t sz=file.size();
-   return sz>4 && file[sz-4]=='.' && file[sz-3]=='x' && file[sz-2]=='p'
-   	&& file[sz-1]=='m';
+void Gtk_Toolbar::PushToolBarElem(const Widget &w,CxxFile &f, 
+			const std::string &instance,const Widget &tb) const
+{ const ChildParamList ch=w.get_Child_params();
+  if (ch.getBoolProperty("new_group"))
+  {  f.Statement() << instance << "tools().push_back(" 
+  	<< Gtk_Button::ToolBarElemPrefix() << "Space())";
+  }
+  if (IsSubwidget(tb,w)==not_Subwidget)
+  {  f.Statement() << instance << "tools().push_back(" 
+  	<< Gtk_Button::ToolBarElemPrefix() << "Element("
+        << Reference(w) 
+        // tooltip_text, tooltip_private_text
+        << "))";
+  }
+  else try
+  { const Gtk_Button &wr=dynamic_cast<const Gtk_Button &>(LookupWriter(w));
+    std::string childInstance=Configuration.InstanceName(w.Name()) + "->";
+
+    if (GTKMM2 && w.hasProperty("icon"))
+    {  const Widget image=wr.get_image(w);
+       LookupWriter(image).CreatePointer(image,f);
+    }
+    wr.ToolBar_vars(w,f); // radio groups etc.
+    
+    f.Declaration();		// *** Necessary to get state right.
+    f.FunctionName() << instance << "tools().push_back";
+    f.FunctionArg();
+    wr.ToolBarElemCreation(w,f);
+    w.markManaged();
+
+    if (w.getProperty("cxx_visibility","private")=="private" && CanBeManaged(w)) 
+       f.Declaration() << wr.TypeName(w) << " *";
+    else f.Statement();
+    f << Configuration.InstanceName(w.Name());
+    f.Assignment() << "static_cast<" << wr.TypeName(w) << " *>(";
+//    if (Configuration.gtkmm_version>=Pkg_Version(1,3,17)) f << '&';
+    f << instance << "tools().back()"<<(GTKMM1 ? "->" : ".")<<"get_widget())";
+  }
+  catch (std::bad_cast &e) 
+  {  assert(!"Internal toolbar children need to derive from Button\n"); }
 }
+
+
+void Gtk_Toolbar::CreatePointer_depending(const Widget &w,CxxFile &f) const
+{  
+   for (Widget::const_iterator i=w.begin();i!=w.end();++i)
+   {  if ((*i).Class()!="Placeholder")
+         PushToolBarElem(*i, f, Configuration.InstanceName(w.Name())+"->", w);
+   }
+   Parent::CreatePointer_depending(w,f);
+}
+
