@@ -1,4 +1,4 @@
-// $Id: tree.cc,v 1.23 2001/10/19 06:59:01 christof Exp $
+// $Id: tree.cc,v 1.27 2002/06/17 07:03:25 christof Exp $
 /*  glade--: C++ frontend for glade (Gtk+ User Interface Builder)
  *  Copyright (C) 1998  Christof Petig
  *  Copyright (C) 1999-2000 Adolf Petig GmbH & Co. KG, written by Christof Petig
@@ -31,8 +31,14 @@ class Gtk_Tree : public Gtk_Container
 	{  return false; }
 	virtual void ConstructionArgs(Widget const &w, CxxFile &f) const
 	{  f.FunctionArg(); }
+	virtual void ApplyPreferences(Tag &t) const;
 };
 
+void Gtk_Tree::ApplyPreferences(Tag &t) const
+{  if (GTKMM2) static_cast<GladeTag*>(&t)->setWidgetType("GtkTreeView");
+   Parent::ApplyPreferences(t);
+}
+
 static Gtk_Tree Gtk_Tree;
 
 const std::string Gtk_Tree::TypeName(const Widget &w) const
@@ -40,7 +46,7 @@ const std::string Gtk_Tree::TypeName(const Widget &w) const
 }
 
 const std::string Gtk_Tree::IncludeName(const Widget &w) const
-{  return "gtk--/tree.h";
+{  return Configuration.GtkmmIncludePath()+"tree.h";
 }
 
 Gtk_Tree::Gtk_Tree()
@@ -49,19 +55,16 @@ Gtk_Tree::Gtk_Tree()
 
 void Gtk_Tree::Configure(const Widget &w, CxxFile &f,const std::string &instance) const
 {  Parent::Configure(w,f,instance);
-   std::string selection_mode(w.getString("selection_mode","GTK_SELECTION_SINGLE"));
-   f.Statement() << instance << "set_selection_mode(" << selection_mode << ')';
-   std::string view_mode(w.getString("view_mode","GTK_TREE_VIEW_LINE"));
-   f.Statement() << instance << "set_view_mode(" << view_mode << ')';
-   bool view_line(w.getBool("view_line",true));
-   f.Statement() << instance << "set_view_lines(" << PRINT_BOOL(view_line) << ')';
+   WriteEnumPropertyNS(w,f,instance, "selection_mode");
+   WriteEnumPropertyNS(w,f,instance, "view_mode");
+   WriteBoolProperty(w,f,instance, "view_line", false, "view_lines");
 }
 
 #if 0
 void Gtk_Tree::Include(CxxFile &f,const Widget &w)
-{  f.Include("gtk--/tree.h");
+{  f.Include(Configuration.GtkmmIncludePath()+"tree.h");
    if (Configuration.sample_code) 
-   {  f.Include("gtk--/treeitem.h");
+   {  f.Include(Configuration.GtkmmIncludePath()+"treeitem.h");
       f.Include("vector");
    }
    Gtk_Container::Include(f,w);
