@@ -1,4 +1,4 @@
-// $Id: window.cc,v 1.30 2002/01/21 17:22:47 christof Exp $
+// $Id: window.cc,v 1.43 2003/03/31 14:09:27 christof Exp $
 /*  glade--: C++ frontend for glade (Gtk+ User Interface Builder)
  *  Copyright (C) 1998  Christof Petig
  *  Copyright (C) 1999-2000 Adolf Petig GmbH & Co. KG, written by Christof Petig
@@ -29,7 +29,7 @@ const std::string Gtk_Window::TypeName(const Widget &w) const
 }
 
 const std::string Gtk_Window::IncludeName(const Widget &w) const
-{  return "gtk--/window.h";
+{  return Configuration.GtkmmIncludePath()+"window.h";
 }
 
 Gtk_Window::Gtk_Window(bool base_class_init)
@@ -40,45 +40,69 @@ Gtk_Window::Gtk_Window(bool base_class_init)
 void Gtk_Window::CInclude(const Widget &w,CxxFile &f) const
 {  Parent::CInclude(w,f)
    if (Configuration.sample_code && !hasSignal("delete_event")) 
-		f.Include("gtk--/main.h");
+		f.Include(Configuration.GtkmmIncludePath()+"main.h");
 }
 #endif
 
 void Gtk_Window::ConstructionArgs(const Widget &w, CxxFile &f) const
-{  const std::string type=w.getString("type","GTK_WINDOW_TOPLEVEL");
-   f.FunctionArg(type);
+{  std::string type=w.getProperty("type","GTK_WINDOW_TOPLEVEL");
+   f.FunctionArg(Gtkmm2Namespace(type));
 }
 
 bool Gtk_Window::NeedExplicitCtor(const Widget &w) const
-{  return w.hasTag("type");
+{  return w.hasProperty("type");
 }
 
 void Gtk_Window::Configure(const Widget &w, CxxFile &f,const std::string &instance) const
 {  Parent::Configure(w,f,instance);
-   std::string title(w.getString("title",""));
-   if (title.size())
-      f.Statement() << instance << "set_title(" << Configuration.Translatable(title) << ')';
-  int default_width=w.getInt("default_width",-1);
-  int default_height=w.getInt("default_height",-1);
-   if (default_width!=-1 || default_height!=-1)
-   {  f.Statement() << instance << "set_default_size(" << default_width << ", " << default_height << ')';
+
+   WriteTranslatableProperty(w,f,instance, "title");
+   WriteIntIntProperty(w,f,instance, "default_width", "default_height", "default_size");
+   WriteBoolProperty(w,f,instance, "modal");
+
+   // or window_position ???
+   WriteEnumPropertyNS(w,f,instance, "position");
+   if (GTKMM2)
+      WriteEnumPropertyNS(w,f,instance, "window_position", true);
+
+#if 1
+   // Compatibility cruft!
+   if (w.hasProperty("allow_shrink") || 
+   	w.hasProperty("allow_grow") ||
+   	w.hasProperty("auto_shrink"))
+   {  bool allow_shrink=w.getBoolProperty("allow_shrink",false);
+      bool allow_grow=w.getBoolProperty("allow_grow",true);
+      bool auto_shrink=w.getBoolProperty("auto_shrink",false);
+      if (allow_shrink || !allow_grow || auto_shrink)
+      {  if (GTKMM2)
+         {  if (!allow_shrink && !allow_grow && auto_shrink)
+               f.Statement() << instance << "set_resizable(false)";
+            else
+            {  std::cerr << w.Name() << ": non recommended window behaviour\n";
+               WriteBoolProperty(w,f,instance, "allow_shrink", true);
+               WriteBoolProperty(w,f,instance, "allow_grow", true);
+//               WriteBoolProperty(w,f,instance, "auto_shrink", true);
+            }
+         }
+         else
+            f.Statement() << instance << "set_policy(" 
+            	<< PRINT_BOOL(allow_shrink)
+           	<< ", " << PRINT_BOOL(allow_grow)
+           	<< ", " << PRINT_BOOL(auto_shrink) << ')';
+      }
    }
-   if (w.getBool("modal",false))
-   {  std::cerr << "To make your window modal you must also call set_transient_for()\n";
-      f.Statement() << instance << "set_modal(" << PRINT_BOOL(true) << ')';
-   }
-   std::string position(w.getString("position",""));
-   if (position.size() && position!="GTK_WIN_POS_NONE")
-      f.Statement() << instance << "set_position(" << position << ')';
-   bool allow_shrink=w.getBool("allow_shrink",false);
-   bool allow_grow=w.getBool("allow_grow",true);
-   bool auto_shrink=w.getBool("auto_shrink",false);
-   if (allow_shrink || !allow_grow || auto_shrink)
-      f.Statement() << instance << "set_policy(" << PRINT_BOOL(allow_shrink)
-        << ", " << PRINT_BOOL(allow_grow)
-        << ", " << PRINT_BOOL(auto_shrink) << ')';
-   if (Configuration.has_accelerators)
-      f.Statement() << instance << "add_accel_group(*(_data->getAccelGroup()))";
+#endif
+   
+   if (GTKMM2)
+      WriteBoolProperty(w,f,instance, "resizable");
+
+   if (w.getBoolProperty("modal",false))
+      std::cerr << "To make your window modal you must also call set_transient_for()\n";
+
+   if (Configuration.has_accelerators && GTKMM1)
+      f.Statement() << instance << "add_accel_group(*(gmm_data->getAccelGroup()))";
+   if (GTKMM2)
+      WriteBoolProperty(w,f,instance, "destroy_with_parent", true);
 }
 
 #if 0
@@ -91,7 +115,7 @@ bool Gtk_Window::SampleCode(CxxFile &f,const Widget &w,enum WriterType::CodePosi
 #endif
       case WriterType::Constructor:
          {  Widget::const_iterator i=w.get_Signals();
-            for (;i!=w.end() && (*i).getString("name","")!="delete_event";++i);
+            for (;i!=w.end() && (*i).getProperty("name","")!="delete_event";++i);
             if (i==w.end())
             {  // 1.1: connect_to_method($1,$2,$3) => $1.connect(SigC::slot($2,$3))
                f << "\tconnect_to_method(destroy, " 
@@ -109,14 +133,13 @@ bool Gtk_Window::SampleCode(CxxFile &f,const Widget &w,enum WriterType::CodePosi
 const Widget Gtk_Window::get_accelgroup(const Widget &w) const
 {  static Tag accelgroup("widget");
    std::string name(Configuration.InstanceName(w.Name())+"_accgrp");
-   if (accelgroup.getString("name")==name)
+   if (Widget(accelgroup).Name()==name)
    {  return Widget(accelgroup);
    }
-   accelgroup=Tag("widget");
-   accelgroup.push_back(Tag("class","GtkAccelGroup"));
-   accelgroup.push_back(Tag("name",name));
-   accelgroup.push_back(Tag(CXX_IS_MANAGED,"true"));
-   return Widget(accelgroup);
+   accelgroup=createWidgetTag("GtkAccelGroup", name);
+   Widget w2(accelgroup);
+   w2.setProperty(CXX_IS_MANAGED);
+   return w2;
 }
 
 void Gtk_Window::CreatePointer(const Widget &w,CxxFile &f) const
@@ -124,12 +147,12 @@ void Gtk_Window::CreatePointer(const Widget &w,CxxFile &f) const
    assert(never_reached);
 }
 
-void Gtk_Window::AdditionalMemberVars(const Widget &w,CxxFile &f,bool container=false) const
+void Gtk_Window::AdditionalMemberVars(const Widget &w,CxxFile &f,bool container) const
 {  if (Configuration.has_accelerators)
    {  Widget w2(get_accelgroup(w));
       f.Private();
       LookupWriter(w2).GHDeclaration(w2,f);
-      f.Declaration("GlademmData *_data");
+      f.Declaration("GlademmData *gmm_data");
    }
    Parent::AdditionalMemberVars(w,f,container);
 }
@@ -144,7 +167,7 @@ void Gtk_Window::ApplyPreferences(Tag &t) const
    if (Configuration.has_accelerators)
    {  Widget w(&t);
       static bool did_warn;
-      if (!did_warn)
+      if (!did_warn && GTKMM1)
       {  did_warn=true;
          std::cerr << "WARNING: accelerators break code compatibility.\n";
       }
@@ -162,19 +185,27 @@ void Gtk_Window::GCInclude(const Widget &w, CxxFile &f) const
 void Gtk_Window::ClassConstructor(const Widget &w, CxxFile &f) const
 {  Parent::ClassConstructor(w,f);
    if (Configuration.has_accelerators)
-   {  Widget w2(get_accelgroup(w));
-      LookupWriter(w2).CreatePointer(w2,f);
-      f.Statement("_data").Assignment("new ").FunctionName("GlademmData")
-      	.FunctionArg(Pointer(w2));
+   {  if (GTKMM1)
+      {  Widget w2(get_accelgroup(w));
+         LookupWriter(w2).CreatePointer(w2,f);
+         f.Statement("gmm_data").Assignment("new ").FunctionName("GlademmData")
+      	   .FunctionArg(Pointer(w2));
+      }
+      else
+      {  f.Statement("gmm_data").Assignment("new ").FunctionName("GlademmData")
+      	   .FunctionArg("get_accel_group()");
+      }
       f.EndLine();
    }
 }
 
 void Gtk_Window::ClassDestructor(const Widget &w, CxxFile &f) const
 {  if (Configuration.has_accelerators)
-   {  Widget w2(get_accelgroup(w));
-      LookupWriter(w2).DestroyPointer(w2,f);
-      f.Statement("delete _data").EndLine();
+   {  if (GTKMM1) 
+      {  Widget w2(get_accelgroup(w));
+         LookupWriter(w2).DestroyPointer(w2,f);
+      }
+      f.Statement("delete gmm_data").EndLine();
    }
    Parent::ClassDestructor(w,f);
 }
